/*
 * Created on 25.10.2004
 *
 */
import interaction.InterpolatorSwitch;

import javax.media.j3d.Alpha;
import javax.media.j3d.Appearance;
import javax.media.j3d.BoundingLeaf;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.Geometry;
import javax.media.j3d.Group;
import javax.media.j3d.Material;
import javax.media.j3d.RotationInterpolator;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Texture;
import javax.media.j3d.TextureAttributes;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.TriangleStripArray;
import javax.vecmath.Color4f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;

import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.Primitive;

/**
 * Eine zu einer Seite aufschwingende T&uuml;r.
 * 
 * @author Martin Hedler
 * @version 0.1a (02.11.2004)
 */
public class Door extends Group
{
    private InterpolatorSwitch iS;

    public Door(float width, float height, Texture tex)
    {
        // Appearance erstellen
        Appearance app = new Appearance();
        app.setMaterial(new Material());

        // Box erstellen
        Box doorObject =
            new Box(
                width / 2,
                height / 2,
                0.02f,
                Box.GENERATE_NORMALS | Box.GENERATE_TEXTURE_COORDS,
                app);

        setAppearance(doorObject, tex);

        //Capabilities setzen
        setCapabilities(doorObject);

        // Hilfobjekte erstellen
        Vector3f translation = new Vector3f();
        Transform3D transformation = new Transform3D();

        // "Tuerangel erstellen"
        translation.set(width / 2, 0, 0);
        transformation.setTranslation(translation);
        TransformGroup centerPos = new TransformGroup(transformation);
        centerPos.addChild(doorObject);

        // Rotations vorbereiten
        TransformGroup rotationGroup = new TransformGroup();
        rotationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        rotationGroup.addChild(centerPos);
        RotationInterpolator interpolator =
            new RotationInterpolator(
                new Alpha(0, 0, 0, 0, 1000, 250, 0, 1000, 250, 0),
                rotationGroup,
                new Transform3D(),
                0,
                (float) Math.toRadians(90));
        interpolator.setSchedulingBoundingLeaf(
            new BoundingLeaf(new BoundingSphere(new Point3d(), 10)));

        centerPos.addChild(interpolator.getSchedulingBoundingLeaf());
        iS = new InterpolatorSwitch(interpolator);
        iS.setSchedulingBoundingLeaf(interpolator.getSchedulingBoundingLeaf());

        this.addChild(rotationGroup);
        this.addChild(interpolator);
        this.addChild(iS);
        
    }

    private void setAppearance(Box doorObject, Texture tex)
    {
        Appearance app =
            (Appearance) doorObject.getAppearance().cloneNodeComponent(false);
        setTexture(tex, app, 1);
        doorObject.getShape(Box.FRONT).setAppearance(app);

        app = (Appearance) doorObject.getAppearance().cloneNodeComponent(false);
        setTexture(tex, app, -1);
        doorObject.getShape(Box.BACK).setAppearance(app);
    }

    private void setCapabilities(Primitive p)
    {
        Shape3D s;
        Geometry g;
        for (int i = 0; i < p.numChildren(); ++i)
        {
            s = p.getShape(i);
            s.setCapability(Shape3D.ALLOW_LOCAL_TO_VWORLD_READ);
            s.setCapability(Shape3D.ALLOW_BOUNDS_READ);
            s.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
            for (int j = 0; j < s.numGeometries(); ++j)
            {
                g = s.getGeometry(j);
                g.setCapability(TriangleStripArray.ALLOW_COUNT_READ);
                g.setCapability(TriangleStripArray.ALLOW_COORDINATE_READ);
                g.setCapability(TriangleStripArray.ALLOW_FORMAT_READ);
                g.setCapability(TriangleStripArray.ALLOW_INTERSECT);
                g.setCapability(TriangleStripArray.ALLOW_NORMAL_READ);
            }
        }
    }

    private void setTexture(Texture tex, Appearance app, float scaleX)
    {
        app.setTexture(tex);

        Transform3D trans = new Transform3D();
        trans.setScale(new Vector3d(scaleX, 1, 1));
        app.setTextureAttributes(
            new TextureAttributes(
                TextureAttributes.MODULATE,
                trans,
                new Color4f(),
                TextureAttributes.FASTEST));
    }

    public void changeState()
    {
        iS.changeState();
    }
}
