import interaction.RotationModificator;
import interaction.TranslationModificator;
import interaction.navigation.GravityBehavior;
import interaction.navigation.GravityKeyBehavior;
import interaction.navigation.GravityMouseBehavior;
import interaction.navigation.ViewPoint;

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.NoSuchElementException;

import javax.media.j3d.AmbientLight;
import javax.media.j3d.Appearance;
import javax.media.j3d.Behavior;
import javax.media.j3d.BoundingLeaf;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.Bounds;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.DirectionalLight;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.IndexedGeometryArray;
import javax.media.j3d.LinearFog;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Texture;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.WakeupCondition;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOnViewPlatformEntry;
import javax.media.j3d.WakeupOnViewPlatformExit;
import javax.media.j3d.WakeupOr;
import javax.swing.JOptionPane;
import javax.vecmath.AxisAngle4f;
import javax.vecmath.Color3f;
import javax.vecmath.Point2f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3f;

import physics.DefaultCollisionHandler;
import physics.DynamicCube;
import physics.DynamicSphere;
import physics.PhysicalBehavior;
import physics.PhysicalEnvironment;

import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.utils.geometry.Sphere;

import construction.Block;
import construction.LinearStair;
import construction.Map;
import construction.Plane;
import construction.RoundStair;
import construction.TextureManager;

/*
 * Created on 19.10.2004
 *
 */

/**
 * @author Martin Hedler
 *
 */
public class Tempel extends Applet
{
    private TransformGroup avatarTrGrp;
    private ViewPoint viewPoint;
    private GravityMouseBehavior mouseB;
    private TranslationModificator trMod;
    private RotationModificator rtMod;

    /**
     * Erstellt das Applet. Ausserdem wird das Universum und alle Sektoren
     * erstellt.
     */
    public Tempel(String texturePath, int baseLevel, boolean physic)
    {
        super();

        Panel3D panel = new Panel3D();

        panel.getUniverse().getViewer().getView().setFieldOfView(
            Math.toRadians(90));
        panel.getUniverse().getViewer().getView().setBackClipDistance(7);

        avatarTrGrp =
            panel.getUniverse().getViewingPlatform().getViewPlatformTransform();

        super.setLayout(new BorderLayout());
        super.add("Center", panel);

        // Texturen erstellen
        System.out.println("Lade Texturen...");
        createTextureSet(texturePath, baseLevel);
        System.out.println("Texturen geladen.");

        BranchGroup contentBranch = new BranchGroup();

        // ViewPointInterpolator erstellen
        viewPoint = new ViewPoint(avatarTrGrp);
        viewPoint.setSchedulingBounds(new BoundingSphere(new Point3d(), 100));
        contentBranch.addChild(viewPoint);

        // Sektoren erstellen
        System.out.println("Erstelle Welt...");
        createWorld(contentBranch);
        System.out.println("Welt erstellt.");

        //Physikobjekte erstellen
        if (physic)
            createPhysics(contentBranch);

        contentBranch.addChild(createLight());
        contentBranch.addChild(createFog());

        Transform3D trans = new Transform3D();
        trans.setTranslation(new Vector3f(0, 3, 5));
        contentBranch.addChild(
            createNavigation(avatarTrGrp, contentBranch, trans));

        createModificationObject(contentBranch);

        contentBranch.compile();

        panel.getUniverse().addBranchGraph(contentBranch);

        System.out.println("Programm gestartet.");
    }

    private BranchGroup createFog()
    {
        BranchGroup bg = new BranchGroup();

        LinearFog fog = new LinearFog(new Color3f(0f, 0f, 0f), 8, 30);
        fog.setInfluencingBounds(
            new BoundingSphere(new Point3d(0, 0, 0), 1000));
        bg.addChild(fog);

        return bg;
    }

    private BranchGroup createNavigation(
        TransformGroup viewPlatformTrGrp,
        BranchGroup terrainGrp,
        Transform3D trans)
    {
        BranchGroup bg = new BranchGroup();

        viewPlatformTrGrp.setTransform(trans);

        // GravityBehavir einschalten
        GravityBehavior fB =
            new GravityBehavior(viewPlatformTrGrp, terrainGrp, false);
        fB.setSchedulingBounds(new BoundingSphere(new Point3d(0, 0, 0), 1000));
        bg.addChild(fB);

        // Tastensteuerung einschalten
        GravityKeyBehavior keyB = new GravityKeyBehavior(fB);
        keyB.setSchedulingBounds(
            new BoundingSphere(new Point3d(0, 0, 0), 1000));
        bg.addChild(keyB);

        //Maussteuerung einschalten
        mouseB = new GravityMouseBehavior(fB);
        mouseB.setSchedulingBounds(
            new BoundingSphere(new Point3d(0, 0, 0), 1000));
        bg.addChild(mouseB);

        return bg;
    }

    private BranchGroup createLight()
    {
        BranchGroup bg = new BranchGroup();

        AmbientLight ambLight = new AmbientLight();
        ambLight.setInfluencingBounds(
            new BoundingSphere(new Point3d(0, 0, 0), 1000));
        ambLight.setEnable(true);

        DirectionalLight dirLight = new DirectionalLight();
        dirLight.setColor(new Color3f(1, 1, 1));
        dirLight.setDirection(-0.35f, -0.4f, -0.3f);
        dirLight.setInfluencingBounds(
            new BoundingSphere(new Point3d(0, 0, 0), 1000));
        dirLight.setEnable(true);

        bg.addChild(ambLight);
        bg.addChild(dirLight);

        return bg;
    }

    private void createPhysics(BranchGroup bg)
    {
        PhysicalEnvironment pE =
            new PhysicalEnvironment(
                new DefaultCollisionHandler(bg, false),
                PhysicalEnvironment.ALL_STEP,
                40,
                1,
                1);

        PhysicalBehavior phyBh = new PhysicalBehavior(pE);
        phyBh.setSchedulingBounds(new BoundingSphere(new Point3d(), 1000));
        bg.addChild(phyBh);

        Transform3D trans = new Transform3D();
        DynamicSphere dynSp1 = new DynamicSphere(0.4f);
        trans.setTranslation(new Vector3f(-2, 8, -5));
        dynSp1.setTransform(trans);
        DynamicSphere dynSp2 = new DynamicSphere(0.4f);
        trans.setTranslation(new Vector3f(-1, 8, -5));
        dynSp2.setTransform(trans);
        DynamicSphere dynSp3 = new DynamicSphere(0.4f);
        trans.setTranslation(new Vector3f(0, 8, -5));
        dynSp3.setTransform(trans);
        DynamicSphere dynSp4 = new DynamicSphere(0.4f);
        trans.setTranslation(new Vector3f(1, 8f, -5));
        dynSp4.setTransform(trans);
        DynamicSphere dynSp5 = new DynamicSphere(0.4f);
        trans.setTranslation(new Vector3f(4.5f, 3.5f, -5));
        dynSp5.setTransform(trans);
        DynamicCube dynCu = new DynamicCube(0.8f);
        trans.setTranslation(new Vector3f(2, 4f, -5));
        dynCu.setTransform(trans);
        dynSp1.setAcceleration(new Vector3f(0, -10f, 0));
        dynSp1.setEnergyPushFactor(0.9f);
        dynSp2.setAcceleration(new Vector3f(0, -10f, 0));
        dynSp2.setEnergyPushFactor(0.9f);
        dynSp3.setAcceleration(new Vector3f(0, -10f, 0));
        dynSp3.setEnergyPushFactor(0.9f);
        dynSp4.setAcceleration(new Vector3f(0, -10f, 0));
        dynSp4.setEnergyPushFactor(0.9f);
        dynSp5.setAcceleration(new Vector3f(0, -10f, 0));
        dynSp5.setEnergyPushFactor(0.9f);
        dynCu.setAcceleration(new Vector3f(0, -10f, 0));
        dynCu.setEnergyPushFactor(0.9f);
        bg.addChild(dynSp1);
        bg.addChild(dynSp2);
        bg.addChild(dynSp3);
        bg.addChild(dynSp4);
        bg.addChild(dynSp5);
        bg.addChild(dynCu);

        DefaultCollisionHandler.setNeededCapabilities(dynSp1);
        DefaultCollisionHandler.setNeededCapabilities(
            ((Sphere) dynSp1.getChild(0)).getShape());
        DefaultCollisionHandler.setNeededCapabilities(dynSp2);
        DefaultCollisionHandler.setNeededCapabilities(
            ((Sphere) dynSp2.getChild(0)).getShape());
        DefaultCollisionHandler.setNeededCapabilities(dynSp3);
        DefaultCollisionHandler.setNeededCapabilities(
            ((Sphere) dynSp3.getChild(0)).getShape());
        DefaultCollisionHandler.setNeededCapabilities(dynSp4);
        DefaultCollisionHandler.setNeededCapabilities(
            ((Sphere) dynSp4.getChild(0)).getShape());
        DefaultCollisionHandler.setNeededCapabilities(dynSp5);
        DefaultCollisionHandler.setNeededCapabilities(
            ((Sphere) dynSp5.getChild(0)).getShape());
        DefaultCollisionHandler.setNeededCapabilities(dynCu);
        for (int i = 0; i < ((Box) dynCu.getChild(0)).numChildren(); ++i)
        {
            DefaultCollisionHandler.setNeededCapabilities(
                ((Box) dynCu.getChild(0)).getShape(i));
        }

        pE.addDynamicTransformGroup(dynSp1);
        pE.addDynamicTransformGroup(dynSp2);
        pE.addDynamicTransformGroup(dynSp3);
        pE.addDynamicTransformGroup(dynSp4);
        pE.addDynamicTransformGroup(dynSp5);
        pE.addDynamicTransformGroup(dynCu);
    }

    private static Texture[] standard_textureSet, stair_TextureSet;
    private static Texture door_Texture;

    private TextureManager manager;

    private void createTextureSet(String texturePath, int baseLevel)
    {
        manager = new TextureManager();
        manager.setDefaultBaseLevel(baseLevel);
        manager.loadImage(
            texturePath + "texture/floors/floor075.jpg",
            "Boden_1");
        
        manager.loadImage(
            texturePath + "texture/floors/floor096.jpg",
            "Decke_1");
        manager.loadImage(
            texturePath + "texture/bricks/brick018.jpg",
            "Mauer_1");
        manager.loadImage(texturePath + "texture/doors/door042.jpg", "Tuer");
        manager.loadImage(
            texturePath + "texture/metal/metal074.jpg",
            "Metalwand");
        manager.loadImage(
            texturePath + "texture/metal/metal065.jpg",
            "Metalboden");

        // Texturarrays anlegen
        try
        {
            standard_textureSet =
                new Texture[] {
                    manager.getTexture("Boden_1"),
                    manager.getTexture("Decke_1"),
                    manager.getTexture("Mauer_1")};
            door_Texture = manager.getTexture("Tuer");
            stair_TextureSet =
                new Texture[] {
                    manager.getTexture("Metalboden"),
                    manager.getTexture("Metalwand")};
        } catch (NoSuchElementException e)
        {
            JOptionPane.showMessageDialog(
                null,
                "Es konnten nicht alle Texturen geladen werden.\n"
                    + "Das Programm wird ohne diese Texturen ausgefhrt.");
        }

    }

    /**
     * Erstellt die Sektoren und f&uuml;gt sie in die angegebene BranchGroup
     * ein.
     * 
     * @param bg Die BranchGroup, in welche die Sektoren eingef&uuml;gt werden
     *           sollen.
     */
    private void createWorld(BranchGroup bg)
    {
        Map map = new Map();

        if (standard_textureSet != null)
            map.setDefaultTextures(standard_textureSet);

        // Eckpunkte der Sektoren erstellen
        Point2f p[] =
            new Point2f[] {
                new Point2f(-20, 20),
                new Point2f(20, 20),
                new Point2f(20, -20),
                new Point2f(-20, -20)};
        Point2f pGang1[] =
            new Point2f[] {
                new Point2f(-2, -20),
                new Point2f(2, -20),
                new Point2f(2, -30),
                new Point2f(-2, -30)};
        Point2f pSaeule1[] =
            new Point2f[] {
                new Point2f(7, 9),
                new Point2f(7, 8),
                new Point2f(8, 7),
                new Point2f(9, 7),
                new Point2f(10, 8),
                new Point2f(10, 9),
                new Point2f(9, 10),
                new Point2f(8, 10)};
        Point2f pSaeule2[] =
            new Point2f[] {
                new Point2f(7, -8),
                new Point2f(7, -9),
                new Point2f(8, -10),
                new Point2f(9, -10),
                new Point2f(10, -9),
                new Point2f(10, -8),
                new Point2f(9, -7),
                new Point2f(8, -7)};
        Point2f pSaeule3[] =
            new Point2f[] {
                new Point2f(-10, -8),
                new Point2f(-10, -9),
                new Point2f(-9, -10),
                new Point2f(-8, -10),
                new Point2f(-7, -9),
                new Point2f(-7, -8),
                new Point2f(-8, -7),
                new Point2f(-9, -7)};
        Point2f pSaeule4[] =
            new Point2f[] {
                new Point2f(-10, 9),
                new Point2f(-10, 8),
                new Point2f(-9, 7),
                new Point2f(-8, 7),
                new Point2f(-7, 8),
                new Point2f(-7, 9),
                new Point2f(-8, 10),
                new Point2f(-9, 10)};
        Point2f pStufe1[] =
            new Point2f[] {
                new Point2f(-15, -15),
                new Point2f(15, -15),
                new Point2f(15, 15),
                new Point2f(-15, 15)};
        Point2f pStufe2[] =
            new Point2f[] {
                new Point2f(-13, -13),
                new Point2f(13, -13),
                new Point2f(13, 13),
                new Point2f(-13, 13)};
        Point2f pStufe3[] =
            new Point2f[] {
                new Point2f(-11, -11),
                new Point2f(11, -11),
                new Point2f(11, 11),
                new Point2f(-11, 11)};
        Point2f pGang2[] =
            new Point2f[] {
                new Point2f(2, -30),
                new Point2f(2, -26),
                new Point2f(20, -26),
                new Point2f(20, -30)};
        Point2f pGang3[] =
            new Point2f[] {
                new Point2f(16, -26),
                new Point2f(16, -20),
                new Point2f(20, -20),
                new Point2f(20, -26)};
        Point2f pEmpore[] =
            new Point2f[] {
                new Point2f(7.1f, -7.1f),
                new Point2f(7.1f, 7.1f),
                new Point2f(-7.1f, 7.1f),
                new Point2f(-7.1f, -7.1f)};
        Point2f pAusschnitt[] =
            new Point2f[] {
                new Point2f(-7, -7),
                new Point2f(-7, 7),
                new Point2f(7, 7),
                new Point2f(7, -7)};
        Point2f pGross[] =
            {
                new Point2f(30, -30),
                new Point2f(30, 30),
                new Point2f(50, 30),
                new Point2f(50, -30)};
        Point2f pTunnel[] =
            {
                new Point2f(20, -2),
                new Point2f(20, 2),
                new Point2f(30, 2),
                new Point2f(30, -2)};
        Point2f pBruecke[] =
            {
                new Point2f(50, -2),
                new Point2f(50, 2),
                new Point2f(30, 2),
                new Point2f(30, -2)};
        Point2f pGeneigteEbene[] =
            {
                new Point2f(5, -6),
                new Point2f(5, -4),
                new Point2f(0, -4),
                new Point2f(0, -6)};
        Point2f pUebergang[] =
            {
                new Point2f(30f, 2),
                new Point2f(30f, -6),
                new Point2f(32f, -6),
                new Point2f(32f, 2)};
        Point2f pDoom1[] =
            {
                new Point2f(50, -10),
                new Point2f(50, 10),
                new Point2f(60, 10),
                new Point2f(60, -1),
                new Point2f(59.8f, -1),
                new Point2f(59.6f, -1.3f),
                new Point2f(59.6f, -1.6f),
                new Point2f(59.8f, -2),
                new Point2f(60, -2),
                new Point2f(60, -10)};
        Point2f pDoom2[] =
            {
                new Point2f(60, -10),
                new Point2f(60, -2),
                new Point2f(60.2f, -2),
                new Point2f(60.4f, -1.6f),
                new Point2f(60.4f, -1.3f),
                new Point2f(60.2f, -1),
                new Point2f(60, -1),
                new Point2f(60, 10),
                new Point2f(65, 10),
                new Point2f(65, -10)};

        // Rotationachsen und Winkel fuer Decke und/oder Boden erstellen
        AxisAngle4f aa = new AxisAngle4f(1, 0, 0, 0);
        AxisAngle4f aUp = new AxisAngle4f(0, 0, 1, (float) Math.toRadians(18));

        // Die Sektoren erstellen
        map.createSector(p, 0, 6); // Startraum
        map.createSector(pStufe1, 0, 0.5f);
        map.createSector(pStufe2, 0.5f, 1);
        map.createSector(pStufe3, 1, 1.5f);
        map.createSector(pSaeule1, 1.5f, 6);
        map.createSector(pSaeule2, 1.5f, 6);
        map.createSector(pSaeule3, 1.5f, 6);
        map.createSector(pSaeule4, 1.5f, 6);
        map.createSector(pGang1, 0, 5); // Hinterer unterer Gang
        map.createSector(pGang2, 2.9f, 11f, aUp, aa);
        // Gang nach oben
        map.createSector(pGang3, 6.1f, 11);
        // hinterer oberer Gang
        map.createSector(p, 6.5f, 11); // Oberer Raum
        map.createSector(pEmpore, 6.5f, 7);
        map.createSector(pAusschnitt, 6f, 7);
        // "Loch" zwischen Raum und Empore
        map.createSector(pGross, -20, 40); // Grosser Raum
        map.createSector(pTunnel, 6.5f, 10);
        // Durchgang zur Bruecke
        map.createSector(pBruecke, 6f, 6.5f);
        map.createSector(pGeneigteEbene, 1.5f, 2.5f, aa, aUp);
        map.createSector(pUebergang, 0.5f, 1f);
        map.createSector(pDoom1, 6.5f, 12);
        map.createSector(pDoom2, 7.5f, 11);

        //Treppen start        
        LinearStair lS = new LinearStair();
        lS.set(2, 12, 0.5f, 1f, 2.5f);
        if (stair_TextureSet != null)
            lS.setTexture(stair_TextureSet);
        Transform3D trans = new Transform3D();
        Vector3f vec = new Vector3f(40, -15.75f, -29);
        trans.rotY(Math.PI);
        trans.setTranslation(vec);
        TransformGroup trGrp = new TransformGroup(trans);
        trGrp.addChild(lS);
        map.addChild(trGrp);

        // Treppe fuer Physik praeperieren
        for (int i = 0; i < lS.numBlocks(); ++i)
        {
            setPhysicCapabilities(lS.getBlock(i));
        }

        RoundStair rS = new RoundStair();
        rS.set(2, 6, (float) Math.toRadians(90), 0.5f, 5, 0);
        if (stair_TextureSet != null)
            rS.setTexture(stair_TextureSet);
        vec.set(46, -18.75f, -26);
        trans.setIdentity();
        trans.setTranslation(vec);
        trGrp = new TransformGroup(trans);
        trGrp.addChild(rS);
        map.addChild(trGrp);

        // Treppe fuer Physik praeperieren
        for (int i = 0; i < rS.numBlocks(); ++i)
        {
            setPhysicCapabilities(rS.getBlock(i));
        }

        rS = new RoundStair();
        rS.set(2, 6, (float) Math.toRadians(90), 0.5f, 5, 8.5f);
        if (stair_TextureSet != null)
            rS.setTexture(stair_TextureSet);
        vec.set(34, -14.5f, -26);
        trans.rotY(Math.PI / 2);
        trans.setTranslation(vec);
        trGrp = new TransformGroup(trans);
        trGrp.addChild(rS);
        map.addChild(trGrp);

        // Treppe fuer Physik praeperieren
        for (int i = 0; i < rS.numBlocks(); ++i)
        {
            setPhysicCapabilities(rS.getBlock(i));
        }

        lS = new LinearStair();
        lS.set(2, 20, 0.5f, 1f, 11);
        if (stair_TextureSet != null)
            lS.setTexture(stair_TextureSet);
        trans = new Transform3D();
        vec = new Vector3f(31, -9.5f, -16);
        trans.rotY(-Math.PI / 2);
        trans.setTranslation(vec);
        trGrp = new TransformGroup(trans);
        trGrp.addChild(lS);
        map.addChild(trGrp);

        // Treppe fuer Physik praeperieren
        for (int i = 0; i < lS.numBlocks(); ++i)
        {
            setPhysicCapabilities(lS.getBlock(i));
        }

        rS = new RoundStair();
        rS.set(2, 4, (float) Math.toRadians(180), 0.5f, 10, 0);
        if (stair_TextureSet != null)
            rS.setTexture(stair_TextureSet);
        vec.set(34, 3.5f, 2);
        trans.rotY(Math.PI);
        trans.setTranslation(vec);
        trGrp = new TransformGroup(trans);
        trGrp.addChild(rS);
        map.addChild(trGrp);

        // Treppe fuer Physik praeperieren
        for (int i = 0; i < rS.numBlocks(); ++i)
        {
            setPhysicCapabilities(rS.getBlock(i));
        }
        // Treppen ende

        // Sektoren fuer Physik praeperieren
        for (Iterator it = map.getSectors(); it.hasNext();)
        {
            setPhysicCapabilities((Block) it.next());
        }

        bg.addChild(map);

        Transform3D transform = new Transform3D();
        transform.setTranslation(new Vector3f(-2, 2.5f, -20));
        TransformGroup doorTrGrp = new TransformGroup(transform);
        door = new Door(4, 5, door_Texture);
        doorTrGrp.addChild(door);
        bg.addChild(doorTrGrp);

        BoundingSphere doorBounds =
            new BoundingSphere(new Point3d(0, 2.5f, -20), 6);
        DoorBehavior dB = new DoorBehavior(doorBounds);
        dB.setSchedulingBounds(doorBounds);
        bg.addChild(dB);
    }

    private void createModificationObject(BranchGroup bg)
    {
        Transform3D trans = new Transform3D();
        trans.setTranslation(new Vector3f(45, -18, -10));
        TransformGroup trGrp = new TransformGroup(trans);
        trGrp.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        trGrp.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);

        trGrp.addChild(new ColorCube(0.4f));

        BoundingLeaf bL = new BoundingLeaf(new BoundingSphere());
        trGrp.addChild(bL);
        bg.addChild(trGrp);

        rtMod = new RotationModificator(trGrp);
        rtMod.setSchedulingBoundingLeaf(bL);
        rtMod.setEnable(false);
        trMod = new TranslationModificator(trGrp);
        trMod.setSchedulingBoundingLeaf(bL);
        trMod.setEnable(false);

        bg.addChild(rtMod);
        bg.addChild(trMod);
    }

    private void setPhysicCapabilities(Block b)
    {
        for (Iterator planeIt = b.getPlanes(); planeIt.hasNext();)
        {
            Plane plane = (Plane) planeIt.next();
            plane.setCapability(Shape3D.ALLOW_LOCAL_TO_VWORLD_READ);
            plane.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
            plane.setCapability(Shape3D.ALLOW_BOUNDS_READ);
            plane.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
            plane.getAppearance().setCapability(Appearance.ALLOW_TEXTURE_WRITE);

            if (plane.numGeometries() > 0)
            {
                IndexedGeometryArray geometry =
                    (IndexedGeometryArray) plane.getGeometry();

                geometry.setCapability(GeometryArray.ALLOW_COUNT_READ);
                geometry.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
                geometry.setCapability(GeometryArray.ALLOW_FORMAT_READ);
                geometry.setCapability(GeometryArray.ALLOW_INTERSECT);
                geometry.setCapability(GeometryArray.ALLOW_NORMAL_READ);
                geometry.setCapability(
                    IndexedGeometryArray.ALLOW_COORDINATE_INDEX_READ);
                geometry.setCapability(
                    IndexedGeometryArray.ALLOW_NORMAL_INDEX_READ);
            }
        }
    }

    Door door;

    public static String getParamValue(String key, String[] args)
    {
        for (int i = 0; i < args.length; ++i)
        {
            if (args[i].startsWith(key))
            {
                if (key.length() < args[i].length())
                {
                    return args[i].substring(key.length());
                } else
                {
                    ++i;
                    if (i < args.length)
                    {
                        return args[i];
                    }
                }
            }
        }

        return null;
    }

    public static void main(String[] args)
    {
        String texturePath = Tempel.getParamValue("-tp", args);
        String baseLevelStr = Tempel.getParamValue("-bl", args);
        String physicsStr = Tempel.getParamValue("-p", args);
        int baseLevel = 2;
        boolean physics = false;

        if (texturePath == null)
        {
            texturePath = new String("./");
        } else
            if (texturePath.endsWith("/") == false)
                texturePath = texturePath + "/";
        if (baseLevelStr != null)
        {
            baseLevel = Integer.parseInt(baseLevelStr);
        }
        if (physicsStr != null)
        {
            physics = Boolean.valueOf(physicsStr).booleanValue();
        }

        Tempel tempel = new Tempel(texturePath, baseLevel, physics);

        MainFrame f =
            new MainFrame(
                tempel,
                640,
                480);
        f.setMenuBar(tempel.createMenuBar());
    }

    public MenuBar createMenuBar()
    {
        MenuBar mB = new MenuBar();

        Menu m1 = new Menu("Gehe zu...");
        m1.add("...Brcke");
        m1.add("...oberen Raum");
        m1.add("...Treppe");
        m1.add("...Startraum");
        m1.addActionListener(new ActionListener()
        {

            Transform3D trans = new Transform3D();
            Vector3f vec = new Vector3f();

            public void actionPerformed(ActionEvent e)
            {
                String s = e.getActionCommand();
                if (s.equals("...Brcke"))
                {
                    vec.set(40, 8.5f, 0);
                    trans.setIdentity();
                    trans.rotY(Math.toRadians(-90));
                    trans.setTranslation(vec);
                } else
                    if (s.equals("...oberen Raum"))
                    {
                        vec.set(18, 8.5f, 18);
                        trans.setIdentity();
                        trans.rotY(Math.toRadians(20));
                        trans.setTranslation(vec);
                    } else
                        if (s.equals("...Treppe"))
                        {
                            vec.set(45, -18.2f, -25);
                            trans.setIdentity();
                            trans.rotY(Math.toRadians(-20));
                            trans.setTranslation(vec);
                        } else
                            if (s.equals("...Startraum"))
                            {
                                vec.set(0, 3.7f, 5);
                                trans.setIdentity();
                                trans.setTranslation(vec);
                            }

                viewPoint.moveTo(trans);
            }
        });
        mB.add(m1);

        Menu m2 = new Menu("Steuerung");
        m2.add("Rotation einschalten");
        m2.add("Translation einschalten");
        m2.add("Maussteuerung einschalten");
        m2.getItem(2).setEnabled(false);
        m2.addActionListener(new ActionListener()
        {

            public void actionPerformed(ActionEvent e)
            {
                String s = e.getActionCommand();

                if (s.equals("Rotation einschalten"))
                {
                    rtMod.setEnable(true);
                    trMod.setEnable(false);
                    mouseB.setEnable(false);
                    ((Menu) e.getSource()).getItem(0).setEnabled(false);
                    ((Menu) e.getSource()).getItem(1).setEnabled(true);
                    ((Menu) e.getSource()).getItem(2).setEnabled(true);
                } else
                    if (s.equals("Translation einschalten"))
                    {
                        rtMod.setEnable(false);
                        trMod.setEnable(true);
                        mouseB.setEnable(false);
                        ((Menu) e.getSource()).getItem(0).setEnabled(true);
                        ((Menu) e.getSource()).getItem(1).setEnabled(false);
                        ((Menu) e.getSource()).getItem(2).setEnabled(true);
                    } else
                        if (s.equals("Maussteuerung einschalten"))
                        {
                            rtMod.setEnable(false);
                            trMod.setEnable(false);
                            mouseB.setEnable(true);
                            ((Menu) e.getSource()).getItem(0).setEnabled(true);
                            ((Menu) e.getSource()).getItem(1).setEnabled(true);
                            ((Menu) e.getSource()).getItem(2).setEnabled(false);
                        }
            }
        });
        mB.add(m2);

        return mB;
    }

    private class DoorBehavior extends Behavior
    {
        WakeupCondition crit;

        public DoorBehavior(Bounds b)
        {
            crit =
                new WakeupOr(
                    new WakeupCriterion[] {
                        new WakeupOnViewPlatformEntry(b),
                        new WakeupOnViewPlatformExit(b)});
        }

        public void initialize()
        {
            this.wakeupOn(crit);
        }

        public void processStimulus(Enumeration e)
        {
            door.changeState();

            this.wakeupOn(crit);
        }
    }
}
