import interaction.PickBehavior;
import interaction.PickListener;
import interaction.navigation.GravityBehavior;
import interaction.navigation.GravityKeyBehavior;

import java.applet.Applet;
import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.NoSuchElementException;

import javax.media.j3d.AmbientLight;
import javax.media.j3d.Behavior;
import javax.media.j3d.BoundingBox;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.DirectionalLight;
import javax.media.j3d.Font3D;
import javax.media.j3d.FontExtrusion;
import javax.media.j3d.PickBounds;
import javax.media.j3d.PickCylinderSegment;
import javax.media.j3d.SceneGraphPath;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Text3D;
import javax.media.j3d.Texture;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOnAWTEvent;
import javax.media.j3d.WakeupOnElapsedFrames;
import javax.media.j3d.WakeupOnTransformChange;
import javax.media.j3d.WakeupOr;
import javax.swing.JOptionPane;
import javax.vecmath.Color3f;
import javax.vecmath.Point2f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;

import physics.DefaultCollisionHandler;
import physics.DynamicSphere;
import physics.PhysicalEnvironment;
import physics.PhysicalThread;

import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.picking.PickResult;
import com.sun.j3d.utils.picking.PickTool;

import construction.Block;
import construction.Map;
import construction.Plane;
import construction.TextureManager;

/*
 * Created on 30.03.2005
 *
 */

/**
 * @author Martin Hedler
 *
 */
public class Test extends Applet
{
    private TextureManager manager;
    private DynamicSphere redDynSp1,
        redDynSp2,
        redDynSp3,
        redDynSp4,
        whiteDynSp;
    private BranchGroup ballGroup;
    private Text3D text;

    private void loadTextures(String texturePath, int baseLevel)
    {
        manager = new TextureManager();
        manager.setDefaultBaseLevel(baseLevel);

        manager.loadImage(
            texturePath + "texture/bricks/brick063.jpg",
            "mauer_1");
        manager.loadImage(
            texturePath + "texture/floors/floor090.jpg",
            "boden_1");
    }

    private Map createRooms()
    {
        // Karte anlegen
        Map map = new Map();

        // Texturen fuer die Raeume setzen
        try
        {
            map.setDefaultTextures(
                new Texture[] {
                    manager.getTexture("boden_1"),
                    manager.getTexture("mauer_1")});
        } catch (NoSuchElementException e)
        {
            JOptionPane.showMessageDialog(
                null,
                "Es konnten nicht alle Texturen geladen werden.\n"
                    + "Das Programm wird ohne diese Texturen ausgefhrt.");
        }

        /* Ecken der Raeume definieren */
        // Punkte des Hauptraums
        Point2f[] pGameRoom =
            new Point2f[] {
                new Point2f(-20, 20),
                new Point2f(20, 20),
                new Point2f(20, -20),
                new Point2f(-20, -20)};
        // Punkte des Tors
        Point2f[] pGoal =
            new Point2f[] {
                new Point2f(-4, -20),
                new Point2f(4, -20),
                new Point2f(3, -25),
                new Point2f(-3, -25)};

        // Raeume erstellen
        map.createSector(pGameRoom, -2, 10);
        map.createSector(pGoal, -3, 1);

        // Capability-Flags setzen
        for (Iterator it = map.getSectors(); it.hasNext();)
        {
            Block s = (Block) it.next();
            for (Iterator it2 = s.getPlanes(); it2.hasNext();)
            {
                Plane pl = (Plane) it2.next();
                GravityBehavior.setNeededCapabilities(pl);
                DefaultCollisionHandler.setNeededCapabilities(pl);
            }
        }

        return map;
    }

    public BranchGroup createLight()
    {
        BranchGroup bg = new BranchGroup();

        AmbientLight ambL = new AmbientLight();
        ambL.setInfluencingBounds(new BoundingSphere(new Point3d(), 1000));
        ambL.setEnable(true);
        bg.addChild(ambL);

        DirectionalLight dirL =
            new DirectionalLight(
                new Color3f(1f, 1f, 1f),
                new Vector3f(-0.5f, -1, -0.25f));
        dirL.setInfluencingBounds(new BoundingSphere(new Point3d(), 1000));
        dirL.setEnable(true);
        bg.addChild(dirL);

        return bg;
    }

    private BranchGroup createNavigation(
        TransformGroup vpTrGrp,
        BranchGroup navBG)
    {
        BranchGroup bg = new BranchGroup();

        GravityBehavior gB = new GravityBehavior(vpTrGrp, navBG, false);
        gB.setSchedulingBounds(new BoundingSphere());
        bg.addChild(gB);

        GravityKeyBehavior nav = new GravityKeyBehavior(gB);
        nav.setSchedulingBounds(gB.getSchedulingBounds());
        bg.addChild(nav);

        return bg;
    }

    public Test(String texturePath, int baseLevel)
    {
        super();

        // Fenster einrichten und Java~3D initialisieren
        this.setLayout(new BorderLayout());
        Panel3D panel = new Panel3D();
        this.add(panel, "Center");

        panel.getUniverse().getViewer().getView().setFieldOfView(
            Math.toRadians(60));

        BranchGroup contentBranch = new BranchGroup();

        // Texturen laden
        System.out.println("Lade Texturen...");
        loadTextures(texturePath, baseLevel);
        System.out.println("Texturen geladen");

        // Rume erstellen
        contentBranch.addChild(createRooms());

        // Navigation einstellen
        contentBranch.addChild(
            createNavigation(
                panel
                    .getUniverse()
                    .getViewingPlatform()
                    .getViewPlatformTransform(),
                contentBranch));

        // Licht erstellen
        contentBranch.addChild(createLight());

        // Physik anlegen
        contentBranch.addChild(createPhysics(contentBranch));

        // Spiellogik erstellen
        contentBranch.addChild(
            createGameLogic(
                contentBranch,
                panel
                    .getUniverse()
                    .getViewingPlatform()
                    .getViewPlatformTransform()));

        contentBranch.compile();

        panel.getUniverse().addBranchGraph(contentBranch);
    }

    private BranchGroup createGameLogic(
        BranchGroup pBG,
        TransformGroup vpTrGrp)
    {
        BranchGroup bg = new BranchGroup();

        BallHoldingBehavior bHB = new BallHoldingBehavior(whiteDynSp, vpTrGrp);
        bHB.setSchedulingBounds(new BoundingSphere());
        bg.addChild(bHB);

        bg.addChild(createScoreView());

        GameBehavior gB =
            new GameBehavior(
                ballGroup,
                vpTrGrp,
                2,
                whiteDynSp,
                bHB,
                new BoundingBox(
                    new Point3d(-4, -3, -25),
                    new Point3d(4, 1, -20)),
                text);
        gB.setSchedulingBounds(new BoundingSphere());
        gB.setEnable(true);
        bg.addChild(gB);

        // PickBehavior initialisieren
        createPickBehavior(pBG, gB);

        return bg;
    }

    private BranchGroup createPhysics(BranchGroup collGroup)
    {
        BranchGroup bg = new BranchGroup();

        Transform3D transform = new Transform3D();

        ballGroup = new BranchGroup();

        PhysicalEnvironment pE =
            new PhysicalEnvironment(
                new DefaultCollisionHandler(collGroup, false),
                PhysicalEnvironment.SINGLE_STEP,
                40,
                5,
                1f);

        // Spielball
        whiteDynSp = new DynamicSphere(0.4f);
        transform.setTranslation(new Vector3f(0, 2, -2));
        whiteDynSp.setTransform(transform);
        whiteDynSp.setAcceleration(new Vector3f(0, -9f, 0));
        whiteDynSp.setMass(2);
        whiteDynSp.setEnergyPushFactor(0.9f);
        ((Sphere) whiteDynSp.getChild(0))
            .getAppearance()
            .getMaterial()
            .setDiffuseColor(
            new Color3f(1, 1, 1));
        ballGroup.addChild(whiteDynSp);
        // Capability-Flags setzen
        DefaultCollisionHandler.setNeededCapabilities(whiteDynSp);
        DefaultCollisionHandler.setNeededCapabilities(
            whiteDynSp.getSphere().getShape());
        pE.addDynamicTransformGroup(whiteDynSp);

        // Gegner
        redDynSp1 = new DynamicSphere(0.4f);
        transform.setTranslation(new Vector3f(-2, 4, 2));
        redDynSp1.setTransform(transform);
        redDynSp1.setAcceleration(new Vector3f(0, -9, 0));
        redDynSp1.setEnergyPushFactor(0.9f);
        ((Sphere) redDynSp1.getChild(0))
            .getAppearance()
            .getMaterial()
            .setDiffuseColor(
            new Color3f(1, 0, 0));
        ballGroup.addChild(redDynSp1);
        // Capability-Flags setzen
        DefaultCollisionHandler.setNeededCapabilities(redDynSp1);
        DefaultCollisionHandler.setNeededCapabilities(
            ((Sphere) redDynSp1.getChild(0)).getShape());
        pE.addDynamicTransformGroup(redDynSp1);

        // Gegner
        redDynSp2 = new DynamicSphere(0.4f);
        transform.setTranslation(new Vector3f(-2, 4, -2));
        redDynSp2.setTransform(transform);
        redDynSp2.setAcceleration(new Vector3f(0, -9, 0));
        redDynSp2.setEnergyPushFactor(0.9f);
        ((Sphere) redDynSp2.getChild(0))
            .getAppearance()
            .getMaterial()
            .setDiffuseColor(
            new Color3f(1, 0, 0));
        ballGroup.addChild(redDynSp2);
        // Capability-Flags setzen
        DefaultCollisionHandler.setNeededCapabilities(redDynSp2);
        DefaultCollisionHandler.setNeededCapabilities(
            ((Sphere) redDynSp2.getChild(0)).getShape());
        pE.addDynamicTransformGroup(redDynSp2);

        // Gegner
        redDynSp3 = new DynamicSphere(0.4f);
        transform.setTranslation(new Vector3f(2, 4, -2));
        redDynSp3.setTransform(transform);
        redDynSp3.setAcceleration(new Vector3f(0, -9, 0));
        redDynSp3.setEnergyPushFactor(0.9f);
        ((Sphere) redDynSp3.getChild(0))
            .getAppearance()
            .getMaterial()
            .setDiffuseColor(
            new Color3f(1, 0, 0));
        ballGroup.addChild(redDynSp3);
        // Capability-Flags setzen
        DefaultCollisionHandler.setNeededCapabilities(redDynSp3);
        DefaultCollisionHandler.setNeededCapabilities(
            ((Sphere) redDynSp3.getChild(0)).getShape());
        pE.addDynamicTransformGroup(redDynSp3);

        // Gegner
        redDynSp4 = new DynamicSphere(0.4f);
        transform.setTranslation(new Vector3f(2, 4, 2));
        redDynSp4.setTransform(transform);
        redDynSp4.setAcceleration(new Vector3f(0, -9, 0));
        redDynSp4.setEnergyPushFactor(0.9f);
        ((Sphere) redDynSp4.getChild(0))
            .getAppearance()
            .getMaterial()
            .setDiffuseColor(
            new Color3f(1, 0, 0));
        ballGroup.addChild(redDynSp4);
        // Capability-Flags setzen
        DefaultCollisionHandler.setNeededCapabilities(redDynSp4);
        DefaultCollisionHandler.setNeededCapabilities(
            ((Sphere) redDynSp4.getChild(0)).getShape());
        pE.addDynamicTransformGroup(redDynSp4);

        PhysicalThread pT = new PhysicalThread(pE, bg);
        pT.start();

        bg.addChild(ballGroup);

        return bg;
    }

    private BranchGroup createScoreView()
    {
        BranchGroup bg = new BranchGroup();

        text =
            new Text3D(
                new Font3D(
                    new Font("Arial", Font.BOLD, 1),
                    new FontExtrusion()));
        text.setAlignment(Text3D.ALIGN_CENTER);
        text.setCapability(Text3D.ALLOW_STRING_WRITE);

        Transform3D trans = new Transform3D();
        trans.setTranslation(new Vector3f(0, 2.5f, -20 + 0.1f));

        TransformGroup textTrGrp = new TransformGroup(trans);
        textTrGrp.addChild(new Shape3D(text));
        bg.addChild(textTrGrp);

        return bg;
    }

    private void createPickBehavior(BranchGroup bg, GameBehavior gB)
    {
        PickBehavior pB = new PickBehavior(ballGroup, PickTool.BOUNDS, false);
        pB.setSchedulingBounds(new BoundingSphere());
        bg.addChild(pB);

        whiteDynSp.setUserData(new PickListener()
        {
            private GameBehavior gB;

            public PickListener set(GameBehavior gB)
            {
                this.gB = gB;
                return this;
            }

            public void pressed(PickResult pR, MouseEvent event)
            {
                // Ball aufnehmen
                gB.takeBall();
            }

            public void released(PickResult pR, MouseEvent event)
            {
            }
        }
        .set(gB));
    }

    public static String getParamValue(String key, String[] args)
    {
        for (int i = 0; i < args.length; ++i)
        {
            if (args[i].startsWith(key))
            {
                if (key.length() < args[i].length())
                {
                    return args[i].substring(key.length());
                } else
                {
                    ++i;
                    if (i < args.length)
                    {
                        return args[i];
                    }
                }
            }
        }

        return null;
    }

    public static void main(String[] args)
    {
        String texturePath = Test.getParamValue("-tp", args);
        String baseLevelStr = Test.getParamValue("-bl", args);
        int baseLevel = 2;

        if (texturePath == null)
        {
            texturePath = new String("./");
        } else
            if (texturePath.endsWith("/") == false)
                texturePath = texturePath + "/";
        if (baseLevelStr != null)
        {
            baseLevel = Integer.parseInt(baseLevelStr);
        }

        new MainFrame(new Test(texturePath, baseLevel), 640, 480);
    }

    private class GameBehavior extends Behavior
    {
        private BranchGroup ballGroup;
        private TransformGroup vpTrGrp;
        private PickCylinderSegment pCS;
        private PickBounds pTor;
        private Point3d startPoint, endPoint;
        private Vector3d translationVec;
        private Vector3d playerVector;
        private Transform3D vpTransform;
        private float playerHeight;
        private DynamicSphere ball;
        private BallHoldingBehavior bHBehavior;
        private WakeupOr wakeup;
        private Vector3f speed;
        private boolean away;
        private int inTor;
        private Text3D text;
        private int points;

        public GameBehavior(
            BranchGroup ballGroup,
            TransformGroup vpTrGrp,
            float playerHeight,
            DynamicSphere ball,
            BallHoldingBehavior bHBehavior,
            BoundingBox bBox,
            Text3D text)
        {
            this.ballGroup = ballGroup;
            this.vpTrGrp = vpTrGrp;
            this.playerHeight = playerHeight;
            this.ball = ball;
            this.bHBehavior = bHBehavior;
            this.away = false;
            this.text = text;
            bHBehavior.setEnable(false);

            this.pTor = new PickBounds(bBox);
            inTor = 0;
            this.points = 0;
        }

        /* (non-Javadoc)
         * @see javax.media.j3d.Behavior#initialize()
         */
        public void initialize()
        {
            pCS = new PickCylinderSegment();
            startPoint = new Point3d();
            endPoint = new Point3d();
            vpTransform = new Transform3D();
            translationVec = new Vector3d();
            playerVector = new Vector3d(0, playerHeight, 0);
            speed = new Vector3f();
            text.setString("0");

            wakeup =
                new WakeupOr(
                    new WakeupCriterion[] {
                        new WakeupOnElapsedFrames(1),
                        new WakeupOnAWTEvent(KeyEvent.KEY_PRESSED)});

            this.wakeupOn(wakeup);
        }

        /* (non-Javadoc)
         * @see javax.media.j3d.Behavior#processStimulus(java.util.Enumeration)
         */
        public void processStimulus(Enumeration enum)
        {
            while (enum.hasMoreElements())
            {
                WakeupCriterion crit = (WakeupCriterion) enum.nextElement();

                if (crit instanceof WakeupOnElapsedFrames)
                {
                    processFrame();
                } else
                {
                    AWTEvent[] events = ((WakeupOnAWTEvent) crit).getAWTEvent();
                    for (int i = 0; i < events.length; ++i)
                    {
                        processAWTEvent(events[i]);
                    }
                }
            }

            this.wakeupOn(wakeup);
        }

        private void processAWTEvent(AWTEvent event)
        {
            if (event.getID() == KeyEvent.KEY_PRESSED)
            {
                if (((KeyEvent) event).getKeyCode() == KeyEvent.VK_SPACE)
                {
                    speed.set(0, 3f, -6f);

                    // Orientierung des Spieler ermitteln
                    vpTrGrp.getTransform(vpTransform);
                    vpTransform.transform(speed);

                    // Ball schiessen
                    ball.setSpeed(speed);
                    speed.set(0, 0, 0);
                    ball.setAngleSpeed(speed);
                    bHBehavior.setEnable(false);
                    ball.setSleep(false);
                } else
                    if (((KeyEvent) event).getKeyCode() == KeyEvent.VK_CONTROL)
                    {
                        speed.set(0, 0f, -6f);

                        ball.getTransform(vpTransform);
                        vpTransform.get(translationVec);
                        translationVec.y -= 0.8f;
                        vpTransform.setTranslation(translationVec);
                        ball.setTransform(vpTransform);

                        // Orientierung des Spieler ermitteln
                        vpTrGrp.getTransform(vpTransform);
                        vpTransform.transform(speed);

                        // Ball schiessen
                        ball.setSpeed(speed);
                        speed.set(0, 0, 0);
                        ball.setAngleSpeed(speed);
                        bHBehavior.setEnable(false);
                        ball.setSleep(false);
                    }
            }
        }

        private void processFrame()
        {
            // Position des Spieler ermitteln
            vpTrGrp.getTransform(vpTransform);
            vpTransform.get(translationVec);
            startPoint.set(translationVec);
            endPoint.sub(startPoint, playerVector);

            // Picken auf Spieler
            pCS.set(startPoint, endPoint, 1);
            SceneGraphPath sGP[] = ballGroup.pickAll(pCS);

            if (sGP != null)
                if (sGP.length > 0)
                {
                    for (int i = 0; i < sGP.length; ++i)
                    {
                        // Wurde der Ball gefangen?
                        if (sGP[i].getNode(0) == ball)
                        {
                            takeBall();
                        } else
                        {
                            // Von gegnerischen Ball getroffen
                            points -= 1;
                            int showPoints = inTor * 100 + points;
                            text.setString(Integer.toString(showPoints));
                        }
                    }
                } else
                {
                    away = false;
                }

            // Picken auf Tor
            sGP = ballGroup.pickAll(pTor);

            int currInTor = 0;

            if (sGP != null)
                if (sGP.length > 0)
                {
                    for (int i = 0; i < sGP.length; ++i)
                    {
                        // Befindet sich ein gegnerischer Ball im Tor?
                        if (sGP[i].getNode(0) != ball)
                        {
                            //Tor werten
                            ++currInTor;
                        }
                    }
                }

            if (currInTor != inTor)
            {
                int showPoints = currInTor * 100 + points;
                text.setString(Integer.toString(showPoints));

                inTor = currInTor;
            }
        }

        public void takeBall()
        {
            if (away == false)
            {
                // Ball aus dem Spiel nehmen
                ball.setSleep(true);
                bHBehavior.activate();
                away = true;
            }
        }
    }

    private class BallHoldingBehavior extends Behavior
    {
        private TransformGroup ballTrGrp;
        private TransformGroup vpTrGrp;
        private WakeupOnTransformChange wakeup;
        private Transform3D vpTransform;
        private Transform3D vpToBallTransform;

        public BallHoldingBehavior(
            TransformGroup ballTrGrp,
            TransformGroup vpTrGrp)
        {
            this.ballTrGrp = ballTrGrp;
            this.vpTrGrp = vpTrGrp;
        }

        /* (non-Javadoc)
         * @see javax.media.j3d.Behavior#initialize()
         */
        public void initialize()
        {
            wakeup = new WakeupOnTransformChange(vpTrGrp);
            vpTransform = new Transform3D();

            vpToBallTransform = new Transform3D();
            vpToBallTransform.setTranslation(new Vector3d(0, -0.5f, -1.5f));

            this.wakeupOn(wakeup);
        }

        /* (non-Javadoc)
         * @see javax.media.j3d.Behavior#processStimulus(java.util.Enumeration)
         */
        public void processStimulus(Enumeration arg0)
        {
            vpTrGrp.getTransform(vpTransform);

            // Transformation des Balls berechnen und setzen
            vpTransform.mul(vpToBallTransform);
            ballTrGrp.setTransform(vpTransform);

            this.wakeupOn(wakeup);
        }

        public void activate()
        {
            this.setEnable(true);

            // Transformation des Balls berechnen und setzen
            vpTrGrp.getTransform(vpTransform);
            vpTransform.mul(vpToBallTransform);
            ballTrGrp.setTransform(vpTransform);
        }
    }
}
