/*
 * Created on 17.03.2005
 *
 */
package construction;

import java.util.HashMap;
import java.util.NoSuchElementException;

import javax.media.j3d.Texture;

import com.sun.j3d.utils.image.TextureLoader;

/**
 * Erm&ouml;glicht die zentrale Verwaltung von Texturen.
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/24)
 */
public class TextureManager
{
    /** In dieser Liste werden die Texturen gespeichert. */
    private HashMap map;

    /**
     * Das Standardlevel, welches bei allen neu geladenen Texturen gesetzt
     * wird.
     */
    private int defaultBaseLevel;

    /**
     * Erstellt den TextureManager, mit dem Stanardtexturlevel 0.
     */
    public TextureManager()
    {
        map = new HashMap();
        this.defaultBaseLevel = 0;
    }

    /**
     * L&auml;dt die mit durch den Pfad angegebene Textur und speichert sie
     * unter dem Namen <code>key</code> ab. Das baseLevel der Textur wird auf
     * den eingestellten Standardwert gesetzt. 
     * 
     * @param fileName Pfadname der Textur.
     * @param key Name, unter welchem die Textur gespeichert wird.
     */
    public void loadImage(String fileName, String key)
    {
        // Dieser Aufruf blockiert, bis das Bild geladen wurde
        TextureLoader loader =
            new TextureLoader(
                fileName,
                TextureLoader.GENERATE_MIPMAP | TextureLoader.BY_REFERENCE,
                null);
        Texture tex = loader.getTexture();

        if (tex != null)
        {
            int level =
                defaultBaseLevel <= tex.getMaximumLevel()
                    ? defaultBaseLevel
                    : tex.getMaximumLevel();
            tex.setBaseLevel(level);
        }

        // Textur in Liste eintragen
        map.put(key, tex);
    }

    /**
     * Liefert das zu dem Namen geh&ouml;rende {@linkplain Texture} Objekt
     * zur&uuml;ck. 
     * 
     * @param key Der Name der gesuchten Textur.
     * @return Die zu dem angegebenen Namen geh&ouml;rende Textur.
     * @throws NoSuchElementException Wenn keine Textur zu dem angegebenen Namen
     *         existiert.
     */
    public Texture getTexture(String key) throws NoSuchElementException
    {
        Object obj = map.get(key);
        if (obj != null)
            return (Texture) obj;
        else
            throw new NoSuchElementException();
    }

    /**
     * Setzt das <code>baseLevel</code> welches jeder neu geladenen Textur
     * zugewiesen wird.
     * 
     * @param baseLevel Das Standardlevel.
     * @see Texture#setBaseLevel(int)
     */
    public void setDefaultBaseLevel(int baseLevel)
    {
        this.defaultBaseLevel = baseLevel;
    }
}