/*
 * Created on 16.03.2005
 *
 */
package interaction;

import java.awt.AWTEvent;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.Enumeration;

import javax.media.j3d.Behavior;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.Node;
import javax.media.j3d.NodeReferenceTable;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.WakeupCondition;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOnAWTEvent;
import javax.media.j3d.WakeupOr;
import javax.vecmath.Vector3f;

/**
 * Erm&ouml;glicht das Verschieben eines Objektes durch die Mausbewegungen des
 * Nutzers.<p/>
 * 
 * Bei gedr&uuml;ckter linker Maustaste kann das Objekt in der XY-Ebene
 * verschoben werden. Bei gedr&uuml;ckter rechter Maustaste kann das Objekt in
 * der XZ-Ebene verschoben werden.
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/25)
 */
public class TranslationModificator extends Behavior
{
    private TransformGroup target;
    private Transform3D deltaTransform, srcTransform;

    private WakeupCriterion wakeupMP;
    private WakeupCondition wakeupMDR;
    private int movingPlane;
    private float scalingFactor;

    private static final int XY_PLANE = 0;
    private static final int XZ_PLANE = 1;

    private Point lastPoint;
    private Vector3f movingVector;

    /**
     * Erstellt einen TranslationModificator, welcher die TransformGroup
     * <code>target</code> manipuliert.
     * 
     * @param target Die TransformGroup, welche manipuliert werden soll.
     */
    public TranslationModificator(TransformGroup target)
    {
        super();

        this.target = target;
        deltaTransform = new Transform3D();
        srcTransform = new Transform3D();
        this.movingPlane = XY_PLANE;
        scalingFactor = 0.001f;
        movingVector = new Vector3f();

        wakeupMP = new WakeupOnAWTEvent(MouseEvent.MOUSE_PRESSED);
        wakeupMDR =
            new WakeupOr(
                new WakeupCriterion[] {
                    new WakeupOnAWTEvent(MouseEvent.MOUSE_RELEASED),
                    new WakeupOnAWTEvent(MouseEvent.MOUSE_DRAGGED)});
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Behavior#initialize()
     */
    public void initialize()
    {
        this.wakeupOn(wakeupMP);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Behavior#processStimulus(java.util.Enumeration)
     */
    public void processStimulus(Enumeration enum)
    {
        WakeupCondition wkco = wakeupMP;
        
        for (; enum.hasMoreElements();)
        {
            AWTEvent[] events =
                ((WakeupOnAWTEvent) enum.nextElement()).getAWTEvent();
            for (int i = 0; i < events.length; ++i)
            {
                MouseEvent mouseEvent = (MouseEvent) events[i];

                switch (mouseEvent.getID())
                {
                    case MouseEvent.MOUSE_PRESSED :
                        start(mouseEvent);
                        wkco = wakeupMDR;
                        break;
                    case MouseEvent.MOUSE_DRAGGED :
                        move(mouseEvent);
                    wkco = wakeupMDR;
                        break;
                }
            }
        }
        this.wakeupOn(wkco);
    }

    /**
     * Startet die Manipulation durch Speichern des Startpunktes des Mauscursors
     * und durch Ermittlung der Verschiebungsebene.
     *  
     * @param e Das ausl&ouml;sende Mausereignis.
     */
    private void start(MouseEvent e)
    {
        lastPoint = e.getPoint();
        if (e.getButton() == MouseEvent.BUTTON1)
            movingPlane = XY_PLANE;
        else
            if (e.getButton() == MouseEvent.BUTTON3)
                movingPlane = XZ_PLANE;
    }

    /**
     * Manipuliert die TransformGroup (Verschiebung)
     * 
     * @param e Das ausl&ouml;sende Mausereignis.
     */
    private void move(MouseEvent e)
    {
        Point currentPoint = e.getPoint();

        switch (movingPlane)
        {
            case XY_PLANE :
                movingVector.set(
                    currentPoint.x - lastPoint.x,
                    lastPoint.y - currentPoint.y,
                    0);
                break;
            case XZ_PLANE :
                movingVector.set(
                    currentPoint.x - lastPoint.x,
                    0,
                    currentPoint.y - lastPoint.y);
                break;
        }

        Transform3D localToVW = new Transform3D();

        ((Canvas3D) e.getSource()).getImagePlateToVworld(localToVW);

        localToVW.transform(movingVector);
        movingVector.scale(scalingFactor);

        lastPoint = currentPoint;
        deltaTransform.setIdentity();
        deltaTransform.setTranslation(movingVector);

            target.getTransform(srcTransform);
            deltaTransform.mul(srcTransform);
            target.setTransform(deltaTransform);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.SceneGraphObject#updateNodeReferences(javax.media.j3d.NodeReferenceTable)
     */
    public void updateNodeReferences(NodeReferenceTable arg0)
    {
        super.updateNodeReferences(arg0);

        this.target = (TransformGroup) arg0.getNewObjectReference(this.target);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Node#cloneNode(boolean)
     */
    public Node cloneNode(boolean arg0)
    {
        TranslationModificator newTrMod =
            new TranslationModificator(target);
        newTrMod.duplicateNode(this, arg0);
        return newTrMod;
    }
}
