package interaction.navigation;
import java.awt.AWTEvent;
import java.awt.event.KeyEvent;
import java.util.Enumeration;

import javax.media.j3d.Behavior;
import javax.media.j3d.Node;
import javax.media.j3d.NodeReferenceTable;
import javax.media.j3d.WakeupCondition;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOnAWTEvent;
import javax.media.j3d.WakeupOr;

/*
 * Created on 19.10.2004
 *
 */

/**
 * Erm&ouml;glicht die Steuerung der <code>ViewPlatform</code> oder eines
 * anderen Objekts mit der Tastatur. Verwendet werden die Pfeiltasten.
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/26)
 */
public class GravityKeyBehavior extends Behavior
{
    //TODO Tasten konfigurierbar gestalten.

    private WakeupCondition crit;
    private GravityBehavior partner;

    private boolean keyUpIsPressed,
        keyDownIsPressed,
        keyLeftIsPressed,
        keyRightIsPressed;

    /**
     * Erstellt eine Instanz der Klasse.
     * 
     * @param fB Das {@linkplain GravityBehavior} Objekt, welches die
     *        gew&uuml;nschte <code>TransformGroup</code> manipuliert.
     */
    public GravityKeyBehavior(GravityBehavior fB)
    {
        partner = fB;

        crit =
            new WakeupOr(
                new WakeupCriterion[] {
                    new WakeupOnAWTEvent(KeyEvent.KEY_PRESSED),
                    new WakeupOnAWTEvent(KeyEvent.KEY_RELEASED)});
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Behavior#initialize()
     */
    public void initialize()
    {
        this.wakeupOn(crit);
    }

    private void proccessKeyEvent(AWTEvent[] events)
    {
        KeyEvent key;
        boolean newValue = false;

        for (int i = 0; i < events.length; ++i)
        {
            if (events[i] instanceof KeyEvent)
            {
                key = (KeyEvent) events[i];
                switch (key.getID())
                {
                    case KeyEvent.KEY_PRESSED :
                        newValue = true;
                        break;
                    case KeyEvent.KEY_RELEASED :
                        newValue = false;
                        break;
                    default :
                        assert false;
                }

                switch (key.getKeyCode())
                {
                    case KeyEvent.VK_UP :
                        keyUpIsPressed = newValue;
                        break;
                    case KeyEvent.VK_DOWN :
                        keyDownIsPressed = newValue;
                        break;
                    case KeyEvent.VK_LEFT :
                        keyLeftIsPressed = newValue;
                        break;
                    case KeyEvent.VK_RIGHT :
                        keyRightIsPressed = newValue;
                        break;
                }

                float linSpeed = 0;
                float angleSpeed = 0;

                if (keyUpIsPressed)
                {
                    linSpeed += 0.015f;
                }
                if (keyDownIsPressed)
                {
                    linSpeed -= 0.015f;
                }
                if (keyLeftIsPressed)
                {
                    angleSpeed += 15f;
                }
                if (keyRightIsPressed)
                {
                    angleSpeed -= 15f;
                }

                if (linSpeed > 0)
                    this.partner.startMovingForward();
                else
                    if (linSpeed < 0)
                        this.partner.startMovingBackward();
                    else
                        if (linSpeed == 0)
                            this.partner.endMoving();
                this.partner.setAngleSpeed(angleSpeed);
            }
        }
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Behavior#processStimulus(java.util.Enumeration)
     */
    public void processStimulus(Enumeration criteria)
    {
        for (; criteria.hasMoreElements();)
        {
            WakeupCriterion crit = (WakeupCriterion) criteria.nextElement();
            if (crit instanceof WakeupOnAWTEvent)
            {
                proccessKeyEvent(
                    (AWTEvent[]) ((WakeupOnAWTEvent) crit).getAWTEvent());
            }
        }

        this.wakeupOn(crit);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.SceneGraphObject#updateNodeReferences(javax.media.j3d.NodeReferenceTable)
     */
    public void updateNodeReferences(NodeReferenceTable arg0)
    {
        super.updateNodeReferences(arg0);

        partner = (GravityBehavior) arg0.getNewObjectReference(partner);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Node#cloneNode(boolean)
     */
    public Node cloneNode(boolean arg0)
    {
        GravityKeyBehavior newGrKBeh = new GravityKeyBehavior(partner);
        newGrKBeh.duplicateNode(this, arg0);
        return newGrKBeh;
    }
}