package interaction.navigation;
import java.awt.AWTEvent;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.Enumeration;

import javax.media.j3d.Behavior;
import javax.media.j3d.Node;
import javax.media.j3d.NodeReferenceTable;
import javax.media.j3d.WakeupCondition;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOnAWTEvent;
import javax.media.j3d.WakeupOr;

/*
 * Created on 19.10.2004
 *
 */

/**
 * Erm&ouml;glicht die Steuerung der <code>ViewPlatform</code> oder eines
 * anderen Objekts mit der Maus. Die Mausbewegung bei gedr&uuml;ckter Maustaste
 * bestimmt die Richtung und Geschwindigkeit der Bewegung.
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/26)
 */
public class GravityMouseBehavior extends Behavior
{
    private WakeupCondition crit;
    private GravityBehavior partner;
    private Point startPoint;
    private boolean mousePressed;

    /**
     * Erstellt eine Instanz der Klasse.
     * 
     * @param fB Das {@linkplain GravityBehavior} Objekt, welches die
     *        gew&uuml;nschte <code>TransformGroup</code> manipuliert.
     */
    public GravityMouseBehavior(GravityBehavior fB)
    {
        partner = fB;
        mousePressed = false;

        crit =
            new WakeupOr(
                new WakeupCriterion[] {
                    new WakeupOnAWTEvent(MouseEvent.MOUSE_PRESSED),
                    new WakeupOnAWTEvent(MouseEvent.MOUSE_RELEASED),
                    new WakeupOnAWTEvent(MouseEvent.MOUSE_DRAGGED)});
    }

    /**
     * Wertet die &uuml;bergebenen Ereignisse aus und setzt entsprechend die
     * Geschwindigkeit der Bewegung.
     * 
     * @param events Die auszuwertenden Ereignisse.
     */
    private void proccessMouseEvent(AWTEvent[] events)
    {
        MouseEvent mouse;

        for (int i = 0; i < events.length; ++i)
        {
            if (events[i] instanceof MouseEvent)
            {
                mouse = (MouseEvent) events[i];
                switch (mouse.getID())
                {
                    case MouseEvent.MOUSE_PRESSED :
                        startPoint = mouse.getPoint();
                        mousePressed = true;
                        break;
                    case MouseEvent.MOUSE_RELEASED :
                        this.partner.setLinearSpeed(0);
                        this.partner.setAngleSpeed(0);
                        mousePressed = false;
                        break;
                    default :
                        assert false;
                }

                if (mousePressed)
                {
                    if (mouse.getID() == MouseEvent.MOUSE_DRAGGED)
                    {
                        Point actPoint = mouse.getPoint();
                        this.partner.setLinearSpeed(
                            (float) (startPoint.y - actPoint.y) / 30f);
                        this.partner.setAngleSpeed(
                            (float) (startPoint.x - actPoint.x) / 37f);
                    }
                }
            }
        }
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Behavior#initialize()
     */
    public void initialize()
    {
        this.wakeupOn(crit);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Behavior#processStimulus(java.util.Enumeration)
     */
    public void processStimulus(Enumeration criteria)
    {
        for (; criteria.hasMoreElements();)
        {
            WakeupOnAWTEvent event = (WakeupOnAWTEvent) criteria.nextElement();
            proccessMouseEvent((AWTEvent[]) event.getAWTEvent());
        }

        this.wakeupOn(crit);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.SceneGraphObject#updateNodeReferences(javax.media.j3d.NodeReferenceTable)
     */
    public void updateNodeReferences(NodeReferenceTable arg0)
    {
        super.updateNodeReferences(arg0);

        partner = (GravityBehavior) arg0.getNewObjectReference(partner);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Node#cloneNode(boolean)
     */
    public Node cloneNode(boolean arg0)
    {
        GravityMouseBehavior newGrMBeh = new GravityMouseBehavior(partner);
        newGrMBeh.duplicateNode(this, arg0);
        return newGrMBeh;
    }
}