/*
 * Created on 25.03.2005
 *
 */
package interaction.navigation;

import java.util.Enumeration;

import javax.media.j3d.Alpha;
import javax.media.j3d.Node;
import javax.media.j3d.RotPosPathInterpolator;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Point3f;
import javax.vecmath.Quat4f;
import javax.vecmath.Vector3f;

/**
 * Erm&ouml;licht eine einfache Pfadinterpolation ausgehend von der aktuellen
 * Transformation der zu manipulierenden <code>TransformGroup</code> zu der
 * Zieltransformation. 
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/26)
 */
public class ViewPoint extends RotPosPathInterpolator
{
    private Quat4f quat[];
    private Point3f pos[];
    private float value[];
    private Transform3D tmpTrans;
    private Vector3f tmpVector;

    /**
     * Erstellt eine Instanz der Klasse.
     * 
     * @param trGrp Die <code>TransformGroup</code> die manipuliert werden soll.
     */
    public ViewPoint(TransformGroup trGrp)
    {
        super(
            new Alpha(1, 2000),
            trGrp,
            new Transform3D(),
            new float[] { 0, 1 },
            new Quat4f[] { new Quat4f(), new Quat4f()},
            new Point3f[] { new Point3f(), new Point3f()});

        this.setEnable(false);

        quat = new Quat4f[2];
        quat[0] = new Quat4f();
        quat[1] = new Quat4f();
        pos = new Point3f[2];
        pos[0] = new Point3f();
        pos[1] = new Point3f();
        value = new float[] { 0, 1 };

        tmpTrans = new Transform3D();
        tmpVector = new Vector3f();
    }

    /**
     * Startet die Interpolation von der aktuellen Position der registrierten
     * <code>TransformGroup</code> aus zur angegebenen Zieltransformation.
     * 
     * @param trans Die Zieltransformation.
     */
    public void moveTo(Transform3D trans)
    {
        // Aktuellen Zustand der TransformGroup ermitteln
        this.target.getTransform(tmpTrans);
        tmpTrans.get(quat[0]);
        tmpTrans.get(tmpVector);
        pos[0].set(tmpVector);

        // Zustand der Zieltransformation ermitteln
        trans.get(quat[1]);
        trans.get(tmpVector);
        pos[1].set(tmpVector);

        this.setPathArrays(value, quat, pos);

        this.getAlpha().setStartTime(System.currentTimeMillis());
        this.setEnable(true);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Behavior#processStimulus(java.util.Enumeration)
     */
    public void processStimulus(Enumeration arg0)
    {
        super.processStimulus(arg0);

        if (this.getAlpha().finished())
        {
            this.setEnable(false);
        }
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Node#cloneNode(boolean)
     */
    public Node cloneNode(boolean arg0)
    {
        ViewPoint newVP = new ViewPoint(this.target);
        newVP.duplicateNode(this, arg0);
        return newVP;
    }
}