/*
 * Created on 12.10.2004
 *
 */
package physics;

import java.util.ArrayList;

import javax.media.j3d.BranchGroup;
import javax.media.j3d.Group;

/**
 * Ein Collisionhandler &uuml;berpr&uuml;ft die bei ihm registrierten Objekte
 * auf Kollisionen mit Objekten einer definierten <code>BranchGroup</code>.
 * 
 * @author Martin Hedler
 * @version 1.0 (05/04/25)
 */
public interface CollisionHandler
{
    /**
     * Liefert eine Liste aller zum Zeitpunkt des Aufrufes bestehenden
     * Kollisionen der registrierten Objekte mit Objekten der
     * registrierten <code>BranchGroup</code> zur&uuml;ck.
     * 
     * @param objList Liste von <code>Group</code> Objekten, welche auf
     *        Kollisionen mit Objekten der registrierten
     *        <code>BranchGroup</code> gepr&uuml;ft werden sollen.
     * @param a Liste, in welche die {@linkplain Collision} Objekte
     *        geschrieben werden.
     * @see Collision
     */
    public void getAllCollisions(ArrayList objList, ArrayList a);
    
    /**
     * Liefert eine Liste aller zum Zeitpunkt des Aufrufes bestehenden
     * Kollisionen des &uuml;bergebenen Objekts mit Objekten der
     * registrierten <code>BranchGroup</code> zur&uuml;ck.
     * 
     * @param grp Die Group, die auf Kollisionen gepr&uuml;ft werden soll.
     * @param a Liste, in welche die {@linkplain Collision} Objekte geschrieben
     *        werden.
     * @see Collision
     */
    public void getCollisions(Group grp, ArrayList a);
    
    /**
     * Setzt die <code>BranchGroup</code>, deren Kindknoten auf Kollisionen mit
     * den in {@linkplain #getAllCollisions(ArrayList, ArrayList)} und
     * {@linkplain #getCollisions(Group, ArrayList)} bergebenen Group Objekten
     * geprft werden sollen.
     * 
     * @param bg Die zu berprfenden BranchGroup.
     */
    public void setRootBranchGroup(BranchGroup bg);
}
