/*
 * Created on 13.10.2004
 *
 */
package physics;

import java.util.Enumeration;

import javax.media.j3d.Behavior;
import javax.media.j3d.Node;
import javax.media.j3d.WakeupOnElapsedFrames;

/**
 * Diese Klasse f&uuml;hrt zyklisch, jedes Frame, einen Simulationschritt durch.
 * 
 * @author Martin Hedler
 * @version 1.0 (05/04/25)
 */
public class PhysicalBehavior extends Behavior
{
    /** Aufwachkriterium: Jedes Frame */
    private WakeupOnElapsedFrames wakeupCondition;

    /** Referenz auf das verwendete {@linkplain PhysicalEnvironment} Objekt. */
    private PhysicalEnvironment pE;

    /**
     * Erstellt eine Instanz dieser Klasse.
     * 
     * @param pE Das {@linkplain PhysicalEnvironment} Objekt, welches f&uuml;r
     *        die Simulation verwendet werden soll.
     * @throws NullPointerException Wenn <code>null</code> &uuml;bergeben wurde.
     */
    public PhysicalBehavior(PhysicalEnvironment pE) throws NullPointerException
    {
        super();

        if (pE == null)
            throw new NullPointerException();

        this.pE = pE;
        this.wakeupCondition = new WakeupOnElapsedFrames(0);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Behavior#initialize()
     */
    public void initialize()
    {
        this.wakeupOn(wakeupCondition);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Behavior#processStimulus(java.util.Enumeration)
     */
    public void processStimulus(Enumeration arg0)
    {
        pE.nextStep();
        this.wakeupOn(wakeupCondition);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Node#cloneNode(boolean)
     */
    public Node cloneNode(boolean arg0)
    {
        PhysicalBehavior newPhyBeh = new PhysicalBehavior(pE);
        newPhyBeh.duplicateNode(this, arg0);
        return newPhyBeh;
    }

}
