/*
 * Created on 13.10.2004
 *
 */
package physics;

import javax.media.j3d.BranchGroup;

/**
 * Ein Thread, der zyklisch in einer Schleife Simulationsschritte ausf&uuml;hrt.
 * 
 * @author Martin Hedler
 * @version 1.0 (05/04/25)
 */
public class PhysicalThread extends Thread
{
    /**
     * Referenz auf das verwendete {@linkplain PhysicalEnvironment} Objekt. 
     */
    private PhysicalEnvironment pE;

    /**
     * Referenz auf die BranchGroup, welche die
     * {@linkplain DynamicTransformGroup} Objekte enth&auml;lt, welche bewegt
     * werden sollen.
     */
    private BranchGroup branchGroup;

    /**
     * Erstellt eine Instanz dieser Klasse.
     * 
     * @param pE Das {@linkplain PhysicalEnvironment} Objekt, welches f&uuml;r
     *        die Simulation verwendet werden soll.
     * @param bg Die BranchGroup, welche die {@linkplain DynamicTransformGroup}
     *        Objekte enth&auml;lt, welche bewegt werden sollen.
     * @throws NullPointerException Wenn einer der Parameter <code>null</code>
     *         ist.
     */
    public PhysicalThread(PhysicalEnvironment pE, BranchGroup bg)
        throws NullPointerException
    {
        super();

        if (pE == null || bg == null)
            throw new NullPointerException();

        this.branchGroup = bg;
        this.pE = pE;
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Behavior#initialize()
     */
    public void run()
    {
        // Erst starten, wenn branchgroup lebt
        while (branchGroup.isLive() == false)
        {
            try
            {
                Thread.sleep(100);
            } catch (InterruptedException e)
            {
                e.printStackTrace();
                break;
            }
        }

        while (super.isAlive() && !this.isInterrupted())
        {
            this.pE.nextStep();

            try
            {
                Thread.sleep(5);
            } catch (InterruptedException e)
            {
                break;
            }
        }
    }
}
