package physics.geometry;
import javax.media.j3d.GeometryArray;

/*
 * Created on 23.06.2004
 *
 */

/**
 * Stellt den einheitlichen Zugriff auf verschiedenen
 * {@linkplain javax.media.j3d.GeometryArray} Typen sicher.
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/25)
 */
public interface GeometryArrayHandler
{
    /**
     * Setzt das {@linkplain GeometryArray} auf welches zugegriffen werden soll.
     * 
     * @param a GeometryArray, auf dessen Polygone zugegriffen werden soll.
     * @throws ClassCastException Wenn ein GeometryArray &uuml;bergeben wurde,
     *            welches von dem GeometryArrayHandler nicht unterst&uuml;tzt
     *            wird.
     */
    public abstract void setGeometryArray(GeometryArray a)
        throws ClassCastException;

    /**
     * Liefert die Anzahl der Polygonen zu&uuml;ck, welche sich in dem
     * referenzierten GeometryArray befinden.
     * 
     * @return Anzahl der Polygone in dem mit
     *         {@linkplain #setGeometryArray(GeometryArray)} referenzierten
     *         GeometryArray.
     */
    public abstract int getPolygonCount() throws IllegalStateException;

    /**
     * Gibt zur&uuml;ck, ob weitere Polygone durch die Funktion
     * {@linkplain #getNextPolygon(Polygon)} abgerufen werden k&ouml;nnen.
     * 
     * @return <code>true</code>, wenn weitere Polygone zur Verf&uuml;gung
     *         stehen, sonst <code>false</code>.
     */
    public abstract boolean hasMorePolygons();

    /**
     * Setzt den Handler zu&uuml;ck, so dass bei einem Aufruf von
     * {@linkplain #getNextPolygon(Polygon)} wieder beim ersten Polygon
     * begonnen wird.
     */
    public abstract void reset();

    /**
     * Liefert das n&auml;chste Polygon zur&uuml;ck.
     * 
     * @param out Das n&auml;chste Polygon.
     * @throws IndexOutOfBoundsException Wenn kein weiteres Polygon mehr
     *            vorhanden ist.
     */
    public abstract void getNextPolygon(Polygon out)
        throws IndexOutOfBoundsException;
}
