/*
 * Created on 25.04.2005
 *
 */
package physics.geometry;

import javax.media.j3d.GeometryArray;
import javax.media.j3d.GeometryStripArray;
import javax.media.j3d.IndexedGeometryArray;
import javax.media.j3d.IndexedTriangleArray;
import javax.media.j3d.IndexedTriangleStripArray;
import javax.media.j3d.QuadArray;
import javax.media.j3d.TriangleArray;
import javax.media.j3d.TriangleFanArray;
import javax.media.j3d.TriangleStripArray;

/**
 * Verwaltet mehrere Instanzen von
 * {@linkplain physics.geometry.GeometryArrayHandler}.
 * 
 * @author Martin Hedler
 * @version (2005/04/25)
 */
public class GeometryArrayHandlerFactory
{
    /** Spezieller GeometryHandler. */
    private TriangleGeometryArrayHandler triangleGeoHandler;

    /** Spezieller GeometryHandler. */
    private QuadGeometryArrayHandler quadGeoHandler;

    /**
     * Spezieller GeometryHandler. 
     */
    private IndexedTriangleGeometryArrayHandler indexedTriangleGeoHandler;

    /** Spezieller GeometryHandler. */
    private TriangleStripGeometryArrayHandler triangleStripGeoHandler;

    /** Spezieller GeometryHandler. */
    private TriangleFanGeometryArrayHandler triangleFanGeoHandler;

    /** Spezieller GeometryHandler. */
    private IndexedTriangleStripGeometryArrayHandler indexedTriangleStripGeoHandler;

    /**
     * Erstellt eine Instanz dieser Klasse und legt f&uuml;r jeden bekannten
     * {@linkplain GeometryArrayHandler} ein entsprechendes Objekt an.
     *
     */
    public GeometryArrayHandlerFactory()
    {
        triangleGeoHandler = new TriangleGeometryArrayHandler();
        quadGeoHandler = new QuadGeometryArrayHandler();
        indexedTriangleGeoHandler = new IndexedTriangleGeometryArrayHandler();
        triangleStripGeoHandler = new TriangleStripGeometryArrayHandler();
        triangleFanGeoHandler = new TriangleFanGeometryArrayHandler();
        indexedTriangleStripGeoHandler =
            new IndexedTriangleStripGeometryArrayHandler();
    }

    /**
     * Gibt den, dem GeometryArray Objekt entsprechenden Handler zur&uuml;ck,
     * der dieses GeometryArray dann verwaltet.
     * Eine zuvor bestehende Beziehung dieses Handlers wird damit
     * aufgel&ouml;st.
     * 
     * @param a Das GeometryArray, welches durch einen Handler verwaltet werden
     *        soll. 
     * @return Der entsprechende GeometryHandler &uuml;ber den nun auf das
     *         GeometryArray Objekt zugegriffen werden kann.
     * @throws IllegalArgumentException Wenn f&uuml;r den Typ des
     *         &uuml;bergebenen GeometryArrays kein GeometryHandler existiert.
     */
    public GeometryArrayHandler getGeometryArrayHandler(GeometryArray a)
        throws IllegalArgumentException
    {
        if (a instanceof GeometryStripArray)
        {
            if (a instanceof TriangleStripArray)
            {
                triangleStripGeoHandler.setGeometryArray(a);
                return triangleStripGeoHandler;
            } else
                if (a instanceof TriangleFanArray)
                {
                    triangleFanGeoHandler.setGeometryArray(a);
                    return triangleFanGeoHandler;
                }
        } else
            if (a instanceof IndexedGeometryArray)
            {
                if (a instanceof IndexedTriangleArray)
                {
                    indexedTriangleGeoHandler.setGeometryArray(a);
                    return indexedTriangleGeoHandler;
                } else
                    if (a instanceof IndexedTriangleStripArray)
                    {
                        indexedTriangleStripGeoHandler.setGeometryArray(a);
                        return indexedTriangleStripGeoHandler;
                    }
            } else
                if (a instanceof TriangleArray)
                {
                    triangleGeoHandler.setGeometryArray(a);
                    return triangleGeoHandler;
                } else
                    if (a instanceof QuadArray)
                    {
                        quadGeoHandler.setGeometryArray(a);
                        return quadGeoHandler;
                    }

        throw new IllegalArgumentException(
            a.getClass() + " wird nicht untersttzt");
    }
}
