package physics.geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.IndexedGeometryArray;
import javax.media.j3d.IndexedTriangleArray;

/*
 * Created on 23.06.2004
 *
 */

/**
 * GeometryArrayHandler f&uuml;r
 * {@linkplain javax.media.j3d.IndexedTriangleArray}
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/25)
 */
public class IndexedTriangleGeometryArrayHandler implements GeometryArrayHandler
{
    private IndexedTriangleArray indexedGeometry;
    private int polygonCount;
    private float coord[];
    private int index[];
    private int vertexCount;
    private int indexVertex;

    /**
     * Erstellt eine leere Instanz dieser Klasse.
     */
    public IndexedTriangleGeometryArrayHandler()
    {
        this.indexedGeometry = null;
        polygonCount = 0;
        coord = new float[3 * 3];
        index = new int[3];
        indexVertex = 0;
    }

    /*
     * (non-Javadoc)
     * @see GeometryArrayHandler#setGeometryArray(javax.media.j3d.GeometryArray)
     */
    public void setGeometryArray(GeometryArray a)
    {
        this.setGeometryArray((IndexedTriangleArray) a);
    }

    /**
     * @see GeometryArrayHandler#setGeometryArray(GeometryArray)
     */
    public void setGeometryArray(IndexedTriangleArray a)
    {
        indexVertex = 0;

        vertexCount = ((IndexedGeometryArray) a).getIndexCount();
        indexedGeometry = a;

        // Anzahl der Polygone im Array ermitteln
        polygonCount = indexedGeometry.getIndexCount() / 3;
    }

    /*
     * (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getPolygonCount()
     */
    public int getPolygonCount()
    {
        return polygonCount;
    }

    /*
     * (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#hasMorePolygons()
     */
    public boolean hasMorePolygons()
    {
        return indexVertex + 3 <= vertexCount;
    }

    /*
     * (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#reset()
     */
    public void reset()
    {
        indexVertex = 0;
    }

    /*
     * (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getNextPolygon(physics.geometry.Polygon)
     */
    public void getNextPolygon(Polygon out) throws IndexOutOfBoundsException
    {
        if (indexVertex + 3 > vertexCount)
        {
            throw new IndexOutOfBoundsException();
        }

        indexedGeometry.getCoordinateIndices(indexVertex, this.index);
        indexedGeometry.getCoordinate(this.index[0], coord);
        out.a.x = coord[0];
        out.a.y = coord[1];
        out.a.z = coord[2];
        indexedGeometry.getCoordinate(this.index[1], coord);
        out.b.x = coord[0];
        out.b.y = coord[1];
        out.b.z = coord[2];
        indexedGeometry.getCoordinate(this.index[2], coord);
        out.c.x = coord[0];
        out.c.y = coord[1];
        out.c.z = coord[2];

        //Normale
        indexedGeometry.getNormalIndices(indexVertex, this.index);
        indexedGeometry.getNormal(this.index[0], coord);
        out.normal.x = coord[0];
        out.normal.y = coord[1];
        out.normal.z = coord[2];
        indexedGeometry.getCoordinate(this.index[1], coord);
        out.normal.x += coord[0];
        out.normal.y += coord[1];
        out.normal.z += coord[2];
        indexedGeometry.getCoordinate(this.index[2], coord);
        out.normal.x += coord[0];
        out.normal.y += coord[1];
        out.normal.z += coord[2];
        out.normal.normalize();

        indexVertex = indexVertex + 3;
    }
}
