package physics.geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.IndexedTriangleStripArray;
import javax.vecmath.Vector3f;

/*
 * Created on 23.06.2004
 *
 */

/**
 * GeometryArrayHandler f&uuml;r
 * {@linkplain IndexedTriangleStripGeometryArrayHandler}
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/25)
 */
public class IndexedTriangleStripGeometryArrayHandler
    implements GeometryArrayHandler
{
    private IndexedTriangleStripArray indexedGeometry;
    private int polygonCount;
    private float coord[];
    private int index[];
    private int stripVertexCount[];
    private int stripCount;
    private int indexVertex, stripStartVertex, indexStrip;
    private Vector3f normal;

    /**
     * Erstellt eine leere Instanz dieser Klasse.
     *
     */
    public IndexedTriangleStripGeometryArrayHandler()
    {
        this.indexedGeometry = null;
        polygonCount = 0;
        coord = new float[3 * 3];
        index = new int[3];
        stripVertexCount = new int[1];
        stripCount = 1;
        indexVertex = 0;
        stripStartVertex = 0;
        indexStrip = 0;
        normal = new Vector3f();
    }

    /*
     * (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#setGeometryArray(javax.media.j3d.GeometryArray)
     */
    public void setGeometryArray(GeometryArray a)
    {
        this.setGeometryArray((IndexedTriangleStripArray) a);
    }

    /**
     * @see GeometryArrayHandler#setGeometryArray(GeometryArray)
     */
    public void setGeometryArray(IndexedTriangleStripArray a)
    {
        indexVertex = 0;
        stripStartVertex = 0;
        indexStrip = 0;

        stripVertexCount = new int[a.getNumStrips()];
        stripCount = a.getNumStrips();
        a.getStripIndexCounts(stripVertexCount);
        indexedGeometry = a;

        // Anzahl der Polygone im Array ermitteln
        polygonCount = indexedGeometry.getIndexCount() - 2 * stripCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getPolygonCount()
     */
    public int getPolygonCount()
    {
        return polygonCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#hasMorePolygons()
     */
    public boolean hasMorePolygons()
    {
        return indexVertex + 3
            <= stripStartVertex + stripVertexCount[indexStrip]
            || indexStrip + 1 < stripCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#reset()
     */
    public void reset()
    {
        indexVertex = 0;
        stripStartVertex = 0;
        indexStrip = 0;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getNextPolygon(physics.geometry.Polygon)
     */
    public void getNextPolygon(Polygon out) throws IndexOutOfBoundsException
    {
        if (indexVertex + 3 > stripStartVertex + stripVertexCount[indexStrip])
        {
            // Auf Ende pruefen
            if (indexStrip + 1 >= stripCount)
            {
                throw new IndexOutOfBoundsException();
            }

            stripStartVertex = stripStartVertex + stripVertexCount[indexStrip];
            ++indexStrip;
            indexVertex = stripStartVertex;
        }

        indexedGeometry.getCoordinateIndices(indexVertex, this.index);
        indexedGeometry.getCoordinate(this.index[0], out.a);
        indexedGeometry.getCoordinate(this.index[1], out.b);
        indexedGeometry.getCoordinate(this.index[2], out.c);

        //Normale
        indexedGeometry.getNormal(this.index[0], out.normal);
        indexedGeometry.getNormal(this.index[1], normal);
        out.normal.add(normal);
        indexedGeometry.getNormal(this.index[2], normal);        
        out.normal.add(normal);
        out.normal.normalize();

        indexVertex = indexVertex + 1;
    }
}