package physics.geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.QuadArray;

/*
 * Created on 23.06.2004
 *
 */

/**
 * GeometryArrayHandler f&uuml;r
 * {@linkplain QuadGeometryArrayHandler}
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/25)
 */
public class QuadGeometryArrayHandler implements GeometryArrayHandler
{
    private QuadArray geometry;
    private int polygonCount;
    private float coord[];
    private int indexPolygon;

    /**
     * Erstellt eine leere Instanz dieser Klasse.
     */
    public QuadGeometryArrayHandler()
    {
        this.geometry = null;
        polygonCount = 0;
        coord = new float[4 * 3];
        indexPolygon = 0;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#setGeometryArray(javax.media.j3d.GeometryArray)
     */
    public void setGeometryArray(GeometryArray a)
    {
        this.setGeometryArray((QuadArray) a);
    }

    /**
     * @see GeometryArrayHandler#setGeometryArray(GeometryArray)
     */
    public void setGeometryArray(QuadArray a)
    {
        indexPolygon = 0;
        geometry = a;

        // Anzahl der Polygone im Array ermitteln
        polygonCount = geometry.getVertexCount() / 2;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getPolygonCount()
     */
    public int getPolygonCount()
    {
        return polygonCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#hasMorePolygons()
     */
    public boolean hasMorePolygons()
    {
        return indexPolygon < polygonCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#reset()
     */
    public void reset()
    {
        indexPolygon = 0;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getNextPolygon(physics.geometry.Polygon)
     */
    public void getNextPolygon(Polygon out) throws IndexOutOfBoundsException
    {
        if (indexPolygon >= polygonCount)
        {
            throw new IndexOutOfBoundsException();
        }

        if (indexPolygon % 2 == 0)
        {
            geometry.getCoordinates(indexPolygon * 2, coord);
            out.a.x = coord[0];
            out.a.y = coord[1];
            out.a.z = coord[2];
            out.b.x = coord[3];
            out.b.y = coord[4];
            out.b.z = coord[5];
            out.c.x = coord[6];
            out.c.y = coord[7];
            out.c.z = coord[8];

            // Normale
            geometry.getNormals(indexPolygon * 2, coord);
            out.normal.x = coord[0];
            out.normal.y = coord[1];
            out.normal.z = coord[2];
            out.normal.x += coord[3];
            out.normal.y += coord[4];
            out.normal.z += coord[5];
            out.normal.x += coord[6];
            out.normal.y += coord[7];
            out.normal.z += coord[8];
        } else
        {
            out.a.x = coord[6];
            out.a.y = coord[7];
            out.a.z = coord[8];
            out.b.x = coord[9];
            out.b.y = coord[10];
            out.b.z = coord[11];
            out.c.x = coord[0];
            out.c.y = coord[1];
            out.c.z = coord[2];
            
            out.normal.x = coord[6];
            out.normal.y = coord[7];
            out.normal.z = coord[8];
            out.normal.x += coord[9];
            out.normal.y += coord[10];
            out.normal.z += coord[11];
            out.normal.x += coord[0];
            out.normal.y += coord[1];
            out.normal.z += coord[2];
        }
        out.normal.normalize();
        
        indexPolygon = indexPolygon + 1;
    }
}
