package physics.geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.TriangleFanArray;

/*
 * Created on 23.06.2004
 *
 */

/**
 * GeometryArrayHandler f&uuml;r
 * {@linkplain TriangleFanGeometryArrayHandler}
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/25)
 */
public class TriangleFanGeometryArrayHandler implements GeometryArrayHandler
{
    private TriangleFanArray geometry;
    private int polygonCount;
    private float coord[], startCoord[], startNormal[];
    private int stripVertexCount[];
    private int stripCount;
    private int indexVertex, stripStartVertex, indexStrip;

    /**
     * Erstellt eine leere Instanz dieser Klasse.
     */
    public TriangleFanGeometryArrayHandler()
    {
        this.geometry = null;
        polygonCount = 0;
        coord = new float[2 * 3];
        startCoord = new float[3];
        startNormal = new float[3];
        stripVertexCount = new int[1];
        stripCount = 1;
        indexVertex = 0;
        stripStartVertex = 0;
        indexStrip = 0;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#setGeometryArray(javax.media.j3d.GeometryArray)
     */
    public void setGeometryArray(GeometryArray a)
    {
        this.setGeometryArray((TriangleFanArray) a);
    }

    /**
     * @see GeometryArrayHandler#setGeometryArray(GeometryArray)
     */
    public void setGeometryArray(TriangleFanArray a)
    {
        indexVertex = 0;
        stripStartVertex = 0;
        indexStrip = 0;

        if (stripCount < a.getNumStrips())
        {
            stripVertexCount = new int[a.getNumStrips()];
        }

        stripCount = a.getNumStrips();
        a.getStripVertexCounts(stripVertexCount);
        geometry = a;

        // Anzahl der Polygone im Array ermitteln
        polygonCount = geometry.getVertexCount() - 2 * stripCount;

        geometry.getCoordinates(0, startCoord);
        geometry.getNormals(0, startNormal);
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getPolygonCount()
     */
    public int getPolygonCount()
    {
        return polygonCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#hasMorePolygons()
     */
    public boolean hasMorePolygons()
    {
        return indexVertex + 3
            <= stripStartVertex + stripVertexCount[indexStrip]
            || indexStrip + 1 < stripCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#reset()
     */
    public void reset()
    {
        indexVertex = 0;
        stripStartVertex = 0;
        indexStrip = 0;
        geometry.getCoordinates(0, startCoord);
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getNextPolygon(physics.geometry.Polygon)
     */
    public void getNextPolygon(Polygon out) throws IndexOutOfBoundsException
    {
        if (indexVertex + 3 > stripStartVertex + stripVertexCount[indexStrip])
        {
            // Auf Ende pruefen
            if (indexStrip + 1 >= stripCount)
            {
                throw new IndexOutOfBoundsException();
            }

            stripStartVertex = stripStartVertex + stripVertexCount[indexStrip];
            ++indexStrip;
            indexVertex = stripStartVertex;
        }

        geometry.getCoordinates(indexVertex, coord);
        out.a.x = startCoord[0];
        out.a.y = startCoord[1];
        out.a.z = startCoord[2];
        out.b.x = coord[0];
        out.b.y = coord[1];
        out.b.z = coord[2];
        out.c.x = coord[3];
        out.c.y = coord[4];
        out.c.z = coord[5];

        // Normale
        geometry.getNormals(indexVertex, coord);
        out.normal.x = startCoord[0];
        out.normal.y = startCoord[1];
        out.normal.z = startCoord[2];
        out.normal.x += coord[0];
        out.normal.y += coord[1];
        out.normal.z += coord[2];
        out.normal.x += coord[3];
        out.normal.y += coord[4];
        out.normal.z += coord[5];
        out.normal.normalize();

        indexVertex = indexVertex + 1;
    }
}
