package physics.geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.TriangleArray;

/*
 * Created on 23.06.2004
 *
 */

/**
 * GeometryArrayHandler f&uuml;r
 * {@linkplain TriangleGeometryArrayHandler}
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/25)
 */
public class TriangleGeometryArrayHandler implements GeometryArrayHandler
{
    private TriangleArray geometry;
    private int polygonCount;
    private float coord[];
    private int vertexCount;
    private int indexVertex;

    /**
     * Erstellt eine leere Instanz dieser Klasse.
     */
    public TriangleGeometryArrayHandler()
    {
        this.geometry = null;
        polygonCount = 0;
        coord = new float[3 * 3];
        indexVertex = 0;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#setGeometryArray(javax.media.j3d.GeometryArray)
     */
    public void setGeometryArray(GeometryArray a)
    {
        this.setGeometryArray((TriangleArray) a);
    }

    /**
     * @see GeometryArrayHandler#setGeometryArray(GeometryArray)
     */
    public void setGeometryArray(TriangleArray a)
    {
        indexVertex = 0;
        vertexCount = a.getVertexCount();
        geometry = a;

        // Anzahl der Polygone im Array ermitteln
        polygonCount = geometry.getVertexCount() / 3;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getPolygonCount()
     */
    public int getPolygonCount()
    {
        return polygonCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#hasMorePolygons()
     */
    public boolean hasMorePolygons()
    {
        return indexVertex + 3 <= vertexCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#reset()
     */
    public void reset()
    {
        indexVertex = 0;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getNextPolygon(physics.geometry.Polygon)
     */
    public void getNextPolygon(Polygon out) throws IndexOutOfBoundsException
    {
        if (indexVertex + 3 > vertexCount)
        {
            throw new IndexOutOfBoundsException();
        }

        geometry.getCoordinates(indexVertex, coord);
        out.a.x = coord[0];
        out.a.y = coord[1];
        out.a.z = coord[2];
        out.b.x = coord[3];
        out.b.y = coord[4];
        out.b.z = coord[5];
        out.c.x = coord[6];
        out.c.y = coord[7];
        out.c.z = coord[8];

        // Normale
        geometry.getNormals(indexVertex, coord);
        out.normal.x = coord[0];
        out.normal.y = coord[1];
        out.normal.z = coord[2];
        out.normal.x += coord[3];
        out.normal.y += coord[4];
        out.normal.z += coord[5];
        out.normal.x += coord[6];
        out.normal.y += coord[7];
        out.normal.z += coord[8];
        out.normal.normalize();

        indexVertex = indexVertex + 3;
    }
}
