package physics.geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.TriangleStripArray;
import javax.vecmath.Vector3f;

/*
 * Created on 23.06.2004
 *
 */

/**
 * GeometryArrayHandler f&uuml;r
 * {@linkplain TriangleStripGeometryArrayHandler}
 * 
 * @author Martin Hedler
 * @version 1.0 (2005/04/25)
 */
public class TriangleStripGeometryArrayHandler implements GeometryArrayHandler
{
    private TriangleStripArray geometry;
    private int polygonCount;
    private float coord[];
    private int stripVertexCount[];
    private int stripCount;
    private int indexVertex, stripStartVertex, indexStrip;
    private Vector3f normal[];

    /**
     * Erstellt eine leere Instanz dieser Klasse.
     */
    public TriangleStripGeometryArrayHandler()
    {
        this.geometry = null;
        polygonCount = 0;
        coord = new float[3 * 3];
        stripVertexCount = new int[1];
        stripCount = 1;
        indexVertex = 0;
        stripStartVertex = 0;
        indexStrip = 0;
        normal = new Vector3f[] {new Vector3f(), new Vector3f(), new Vector3f()};
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#setGeometryArray(javax.media.j3d.GeometryArray)
     */
    public void setGeometryArray(GeometryArray a)
    {
        this.setGeometryArray((TriangleStripArray) a);
    }

    /**
     * @see GeometryArrayHandler#setGeometryArray(GeometryArray)
     */
    public void setGeometryArray(TriangleStripArray a)
    {
        indexVertex = 0;
        stripStartVertex = 0;
        indexStrip = 0;

        if (stripCount < a.getNumStrips())
        {
            stripVertexCount = new int[a.getNumStrips()];
        }

        stripCount = a.getNumStrips();
        a.getStripVertexCounts(stripVertexCount);
        geometry = a;

        // Anzahl der Polygone im Array ermitteln
        polygonCount = geometry.getVertexCount() - 2 * stripCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getPolygonCount()
     */
    public int getPolygonCount()
    {
        return polygonCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#hasMorePolygons()
     */
    public boolean hasMorePolygons()
    {
        return indexVertex + 3
            <= stripStartVertex + stripVertexCount[indexStrip]
            || indexStrip + 1 < stripCount;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#reset()
     */
    public void reset()
    {
        indexVertex = 0;
        stripStartVertex = 0;
        indexStrip = 0;
    }

    /*
     *  (non-Javadoc)
     * @see physics.geometry.GeometryArrayHandler#getNextPolygon(physics.geometry.Polygon)
     */
    public void getNextPolygon(Polygon out) throws IndexOutOfBoundsException
    {
        if (indexVertex + 3 > stripStartVertex + stripVertexCount[indexStrip])
        {
            // Auf Ende pruefen
            if (indexStrip + 1 >= stripCount)
            {
                throw new IndexOutOfBoundsException();
            }

            stripStartVertex = stripStartVertex + stripVertexCount[indexStrip];
            ++indexStrip;
            indexVertex = stripStartVertex;
        }

        geometry.getCoordinates(indexVertex, coord);
        out.a.x = coord[0];
        out.a.y = coord[1];
        out.a.z = coord[2];
        out.b.x = coord[3];
        out.b.y = coord[4];
        out.b.z = coord[5];
        out.c.x = coord[6];
        out.c.y = coord[7];
        out.c.z = coord[8];

        // Normale
        geometry.getNormals(indexVertex, normal);
        out.normal.add(normal[0], normal[1]);
        out.normal.add(normal[2]);
        out.normal.normalize();

        indexVertex = indexVertex + 1;
    }
}
